/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.dua.aggrlzzsdtv.Auftrag
 *
 * de.inovat.dua.aggrlzzsdtv.Auftrag is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.dua.aggrlzzsdtv.Auftrag is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.dua.aggrlzzsdtv.Auftrag; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.dua.aggrlzzsdtv;

//~ JDK IMPORTE ===============================================================

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

//~ KLASSEN ===================================================================

/**
 * Enthält alle Informationen für einen Aggregationsauftrag.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Auftrag {

    /** IntervallBeginn */
    private long _intervallBeginn;

    /** Legt die Intervalldauer fest (MQ und LZZS), für die der Auftrag durchgeführt werden soll. */
    private IntervallDauer _intervallDauer;

    /** IntervallEnde: Intervallbeginn + Intervalllänge (ausschließlich!, also der nächste Intervallbeginn */
    private long _intervallEnde;

    /** Legt fest, ob Auftrag genau einmal OHNE weitere Folgeaufträge abgewickelt werden soll. */
    private boolean _isEinmalAuftrag;

    /** VewLZZS - Verwaltungsobjekt, welches den Auftrag ausführen soll */
    private VewLZZS _vewLZZS;

    /** Zeitpunkt der geplanten Ausführung für diesen Auftrag in Millisekunden seit 1.1.1970 */
    private long _ausfuehrungsZeitPunkt = 0;

    /** Datumsformatierung für Ausgabe toString() */
    SimpleDateFormat _datumsFormatierer = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor, um einen Folgeauftrag auf Basis eines bestehenden Auftrags zu berechnen
     *
     * @param auftrag Aktueller Auftrag, für den der Folgeauftrag bestimmt werden soll
     */
    public Auftrag(Auftrag auftrag) {
        this._vewLZZS         = auftrag.getVewLZZS();
        this._isEinmalAuftrag = auftrag.isEinmalAuftrag();
        this._intervallDauer  = auftrag.getIntervallDauer();

        long     minimaleWarteZeit = auftrag.getAusfuehrungsZeitPunkt() - auftrag.getIntervallEnde();
        Calendar intervallBeginn   = new GregorianCalendar();

        intervallBeginn.setTimeInMillis(auftrag.getIntervallBeginn());

        Calendar intervallEnde = new GregorianCalendar();

        intervallEnde.setTimeInMillis(auftrag.getIntervallEnde());

        switch (_intervallDauer) {
        case DTV_TAG : {
            intervallBeginn.add(Calendar.DAY_OF_YEAR, 1);
            this._intervallBeginn = intervallBeginn.getTimeInMillis();
            intervallEnde.add(Calendar.DAY_OF_YEAR, 1);
            this._intervallEnde = intervallEnde.getTimeInMillis();

            break;
        }

        case DTV_MONAT : {
            intervallBeginn.add(Calendar.MONTH, 1);
            this._intervallBeginn = intervallBeginn.getTimeInMillis();
            intervallEnde.add(Calendar.MONTH, 1);
            this._intervallEnde = intervallEnde.getTimeInMillis();

            break;
        }

        case DTV_QUARTAL : {
            intervallBeginn.add(Calendar.MONTH, 3);
            this._intervallBeginn = intervallBeginn.getTimeInMillis();
            intervallEnde.add(Calendar.MONTH, 3);
            this._intervallEnde = intervallEnde.getTimeInMillis();

            break;
        }

        case DTV_JAHR :
        case MSV_JAHR : {
            intervallBeginn.add(Calendar.YEAR, 1);
            this._intervallBeginn = intervallBeginn.getTimeInMillis();
            intervallEnde.add(Calendar.YEAR, 1);
            this._intervallEnde = intervallEnde.getTimeInMillis();
        }
        }

        this._ausfuehrungsZeitPunkt = this._intervallEnde + minimaleWarteZeit;
    }

    /**
     * Konstruktor, um den initialen Erstauftrag für ein Verwaltungsobjekt und das erste zu berechnende Intervall
     * anzulegen
     *
     * @param ausfuehrungsZeitpunkt Zeitpunkt, zu dem Auftrag frühestens ausgeführt werden soll.
     * @param intervallBeginn       Intervallbeginn (EINSCHLIESSLICH!!) in Millisekunden seit 1.1. 1970
     * @param intervallEnde         Intervallende (AUSSCHLIESSLICH!!) in Millisekunden seit 1.1. 1970
     * @param vewLZZS               Verwaltungsobjekt, welches den Auftrag letztendlich ausführen soll
     * @param isEinmalAuftrag       <code>true</code>, wenn es sich um einen einmalig auszuführenden Auftrag OHNE
     *                              Folgeauftrag handelt, sonst <code>false</code>.
     * @param intervallDauer        Intervalldauer dieses Auftrags (DTV_TAG, DTV_MONAT, DTV_QUARTAL, DTV_JAHR,
     *                              MSV_JAHR)
     */
    public Auftrag(long ausfuehrungsZeitpunkt, long intervallBeginn, long intervallEnde, VewLZZS vewLZZS, boolean isEinmalAuftrag, IntervallDauer intervallDauer) {
        _ausfuehrungsZeitPunkt = ausfuehrungsZeitpunkt;
        _intervallBeginn       = intervallBeginn;
        _intervallEnde         = intervallEnde;
        _vewLZZS               = vewLZZS;
        _isEinmalAuftrag       = isEinmalAuftrag;
        _intervallDauer        = intervallDauer;
    }

    //~ METHODEN ==============================================================

    /**
     * Führt den Auftrag aus, indem am zugeordneten Verwaltungsobjekt die Methode {@link
     * VewLZZS#ausfuehren(de.inovat.dua.aggrlzzsdtv.Auftrag)} aufgerufen wir.
     *
     * @return Meldung mit dem Ergebnis der Ausführung des Auftrags.
     */
    public String ausfuehren() {
        return _vewLZZS.ausfuehren(this);
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert den Zeitpunkt der geplanten Ausführung für diesen Auftrag in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt der geplanten Ausführung für diesen Auftrag in Millisekunden seit 1.1.1970
     */
    public long getAusfuehrungsZeitPunkt() {
        return _ausfuehrungsZeitPunkt;
    }

    /**
     * Liefert den Zeitpunkt des Intervallbeginns (EINSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt des Intervallbeginns (EINSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     */
    public long getIntervallBeginn() {
        return _intervallBeginn;
    }

    /**
     * Liefert die Intervalldauer dieses Auftrags (DTV_TAG, DTV_MONAT, DTV_QUARTAL, DTV_JAHR, MSV_JAHR)
     *
     * @return Intervalldauer dieses Auftrags (DTV_TAG, DTV_MONAT, DTV_QUARTAL, DTV_JAHR, MSV_JAHR)
     */
    public IntervallDauer getIntervallDauer() {
        return _intervallDauer;
    }

    /**
     * Liefert den Zeitpunkt des Intervallendes (AUSSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt des Intervallendes (AUSSCHLIESSLICH!!) in Millisekunden seit 1.1.1970
     */
    public long getIntervallEnde() {
        return _intervallEnde;
    }

    /**
     * Liefert das Verwaltungsobjekt, welches den Auftrag ausführen soll.
     *
     * @return Verwaltungsobjekt, welches den Auftrag ausführen soll.
     */
    public VewLZZS getVewLZZS() {
        return _vewLZZS;
    }

    /**
     * Test, ob es sich um einen Einmalauftrag oder um einen Auftrag mit zyklischer Wiederholung handelt.
     *
     * @return <code>true</code>, wenn es sich um einen einmalig auszuführenden Auftrag OHNE Folgeauftrag handelt, sonst
     *         <code>false</code>.
     */
    public boolean isEinmalAuftrag() {
        return _isEinmalAuftrag;
    }

    //~ METHODEN ==============================================================

    @Override

    /**
     * Gibt die Textrepräsentation für einen Auftrag aus.
     *
     * @return Textrepräsentation für einen Auftrag.
     */
    public String toString() {
	    String ausfuehrung = "DauerAuftrag";

        if (_isEinmalAuftrag) {
            ausfuehrung = "EinmalAuftrag";
        }

        String art = "unbekannt";

        switch (_intervallDauer) {
        case DTV_TAG :
            art = "DTV-Tag";

            break;

        case DTV_MONAT :
            art = "DTV-Monat";

            break;

        case DTV_QUARTAL :
            art = "DTV-Quartal";

            break;

        case DTV_JAHR :
            art = "DTV-Jahr";

            break;

        case MSV_JAHR :
            art = "MSV-Jahr";

            break;
        }

        return String.format("Auftrag (%-12s- %-15s) Ausführung am/um %s: Intervall %s - %s fuer %s", art, ausfuehrung, _datumsFormatierer.format(new Date(_ausfuehrungsZeitPunkt)), _datumsFormatierer.format(new Date(_intervallBeginn)), _datumsFormatierer.format(new Date(_intervallEnde)), _vewLZZS.getPid());
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 29.11.11
