/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.dua.aggrlzzsdtv.VewLZZS
 *
 * de.inovat.dua.aggrlzzsdtv.VewLZZS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.dua.aggrlzzsdtv.VewLZZS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.dua.aggrlzzsdtv.VewLZZS; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.dua.aggrlzzsdtv;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.archive.*;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.sys.funclib.debug.Debug;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Verwaltungsklasse je Aggregationsobjekt (LZZS), welches für genau ein Objekt die Aggregation für einen Auftrag
 * ausführt.
 * <p>
 * Das Verwaltungsobjekt fragt sowohl die Archivdaten ab, berechnet die Aggreagationsdaten, und schreibt das Ergebnis
 * zurück in den Datenverteiler.
 * <p>
 * Weiterhin gibt die Verwaltungsklasse eine aussagekräftige Meldung zurück, die den vollständigen Ablauf der
 * Aggregation beschreibt.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class VewLZZS implements ClientSenderInterface {

    /** DebugLogger für Debug-Ausgaben. */
    private static final Debug debug = Debug.getLogger();

    /** Datumsformattierer  zur Ausgabe von Datum und Uhrzeit. */
    private static SimpleDateFormat datumsFormatierer = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    /** Archivsystem */
    private static ArchiveRequestManager _archiv;
    private static Aspect                _aspDTVJahr;
    private static Aspect                _aspDTVMonat;
    private static Aspect                _aspDTVQuartal;
    private static Aspect                _aspDTVTag;
    private static Aspect                _aspExterneErfassung;
    private static Aspect                _aspMessWertErsetzung;
    private static AttributeGroup        _atgFsDTV;
    private static AttributeGroup        _atgFSVerkehrsDatenLangZeitIntervall;
    private static AttributeGroup        _atgMqDTV;
    private static AttributeGroup        _atgMSV;

    /** Datenverteilerverbindung */
    private static ClientDavInterface _dav;

    /** Die Konfiguration des DaV. */
    private static DataModel _konfiguration;

    //~ FELDER ================================================================

    /** Intervalldauer in Stunden, mit denen die Daten am FS erfasst wurden. */
    private int _intervallDauerInStunden;

    /** Legt fest, ob die DTV-Berechnung aus den erfassten FS-Daten erfolgt oder aus den reimportierten BASt-Band-Daten */
    private boolean _isAggregationAusErfasstenDaten;

    /**
     * Legt fest, ob vor der Aggregation die 8+1-q-Werte aus aus den reimportierten BASt V2004-Werten
     * (atg.verkehrsDatenLangZeitIntervall, asp.messWertErsetzung) übernommen werden sollen. Dies erfolgt aber nur, wenn
     * auch <code>isAggregationAusErfasstenDaten=true</code> ist.
     */
    private boolean _isQWerteAusReimportAtgVerkehrsDatenLangzeitIntervall;

    /** Liste mit den MQ an dieser LZZS */
    private List<Mq> _listeMq;

    /** Das Systemobjekt, für welches diese Verwaltungsklasse die Daten berechnet */
    SystemObject _lzzs;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor der Verwaltungs Klasse.
     *
     * @param dav                            Die Datenverteilerverbindung
     * @param lzzs                           Das Systemobjekt, für welches diese Verwaltungsklasse die Daten berechnen
     *                                       soll
     * @param intervallDauerInStunden        Intervalldauer in Stunden für die erfassten FS-Daten
     * @param isAggregationAusErfasstenDaten <code>true</code>, wenn die Aggregation aus den erfassten FS-Daten erfolgen
     *                                       soll, <code>false</code> wenn sie aus den messwertersetzen Daten (Reimport
     * @param isQWerteAusReimportAtgVerkehrsDatenLangzeitIntervall
     *                                       <code>true</code>, wenn vor der Aggregation die 8+1-q-Werte aus aus den
     *                                       reimportierten BASt V2004-Werten (atg.verkehrsDatenLangZeitIntervall,
     *                                       asp.messWertErsetzung) übernommen werden sollen, sonst <code>false</code>.
     *                                       Dies erfolgt aber nur, wenn auch <code>isAggregationAusErfasstenDaten=true</code>
     *                                       ist.
     */
    public VewLZZS(ClientDavInterface dav, SystemObject lzzs, int intervallDauerInStunden, boolean isAggregationAusErfasstenDaten, boolean isQWerteAusReimportAtgVerkehrsDatenLangzeitIntervall) {
        _dav                                                  = dav;
        _lzzs                                                 = lzzs;
        _intervallDauerInStunden                              = intervallDauerInStunden;
        _isAggregationAusErfasstenDaten                       = isAggregationAusErfasstenDaten;
        _isQWerteAusReimportAtgVerkehrsDatenLangzeitIntervall = isQWerteAusReimportAtgVerkehrsDatenLangzeitIntervall;
        _konfiguration                                        = _dav.getDataModel();
        _archiv                                               = _dav.getArchive();
        _atgFSVerkehrsDatenLangZeitIntervall                  = _konfiguration.getAttributeGroup("atg.verkehrsDatenLangZeitIntervall");
        _atgFsDTV                                             = _konfiguration.getAttributeGroup("atg.verkehrsDatenLangZeitIntervallGeschwindigkeitVerteilungFs");
        _atgMqDTV                                             = _konfiguration.getAttributeGroup("atg.verkehrsDatenLangZeitIntervallGeschwindigkeitVerteilungDtvMq");
        _atgMSV                                               = _konfiguration.getAttributeGroup("atg.verkehrsDatenLangZeitMSV");
        _aspExterneErfassung                                  = _konfiguration.getAspect("asp.externeErfassung");
        _aspMessWertErsetzung                                 = _konfiguration.getAspect("asp.messWertErsetzung");
        _aspDTVTag                                            = _konfiguration.getAspect("asp.agregationDtvTag");
        _aspDTVMonat                                          = _konfiguration.getAspect("asp.agregationDtvMonat");
        _aspDTVQuartal                                        = _konfiguration.getAspect("asp.agregationDtvQuartal");
        _aspDTVJahr                                           = _konfiguration.getAspect("asp.agregationDtvJahr");

        // Auswertequerschnitte, MQ und FS suchen und intern merken
        _listeMq = ermittleMq(_lzzs);

        // Sendeanmeldung für Zielattributgruppen
        try {
            _dav.subscribeSender(this, _lzzs, new DataDescription(_atgMqDTV, _aspDTVTag), SenderRole.source());
            _dav.subscribeSender(this, _lzzs, new DataDescription(_atgMqDTV, _aspDTVMonat), SenderRole.source());
            _dav.subscribeSender(this, _lzzs, new DataDescription(_atgMqDTV, _aspDTVQuartal), SenderRole.source());
            _dav.subscribeSender(this, _lzzs, new DataDescription(_atgMqDTV, _aspDTVJahr), SenderRole.source());
            _dav.subscribeSender(this, _lzzs, new DataDescription(_atgMSV, _aspDTVJahr), SenderRole.source());
        }
        catch (OneSubscriptionPerSendData e) {
            debug.warning(String.format("Doppelanmeldung für [%s] (andere Quelle vorhanden?). Statistikdaten werden für Objekt nicht weiter berücksichtigt.", _lzzs.getPid()), e);
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Berechnet einen gemittelten Prozent-Wert aus zwei Werten
     *
     * @param wert1 Wert 1
     * @param wert2 Wert 2
     *
     * @return Gemittelteter Prozent-Wert aus zwei Werten.
     */
    private long aggregiereProzentWert(long wert1, long wert2) {

        // Beide Werte sind gültig, dann einfach den Mittelwert zurückgeben
        if ((wert1 >= -100000) && (wert2 >= -100000)) {
            return Math.round((float) (wert1 + wert2) / 2.0);
        }

        // Nur ein Wert gültig, dann diesen Wert zurückgeben
        if (wert1 >= -100000) {
            return wert1;
        }

        if (wert2 >= -100000) {
            return wert2;
        }

        // Keine gültigen Werte --> "nicht ermittelbar" zurückgeben
        return -100001;
    }

    /**
     * Berechnet einen gemittelten Prozent-Wert aus einer Liste von Archivdatensätzen für einen Zeitraum (Tag, Monat,
     * Quartal oder Jahr)
     *
     * @param mapArchivDaten Map mit den zeitlich sortierten Archivdatensätzen für einen Zeitraum.
     * @param attributName   Attributname am MQ. Bei FS wird umgerechnet, indem erster Buchstabe klein geschrieben wird
     * @param isMqWert       <code>true</code>, wenn Archivdatensätze aus der MQ ATG bestehen, <code>false</code>, wenn
     *                       Archivdatensätze aus den FS ATG bestehen (Attributnamen beginnen in diesem Fall mit einem
     *                       Kleinbuchstaben).
     *
     * @return Gemittelter Prozent-Wert aus den Archivdaten des Zeitraums für das angegebene Attribut.
     */
    private long aggregiereProzentWertAusArchivDaten(Map<Long, ArchiveData> mapArchivDaten, String attributName, boolean isMqWert) {
        long ProzentWert         = 0;
        int  anzVerfuegbareDaten = 0;

        // Attributname anpassen, erster Buchstabe muss klein geschrieben werden, wenn es keine MQ-ATG-Daten sind
        if (!isMqWert) {
            attributName = attributName.substring(0, 1).toLowerCase() + attributName.substring(1, attributName.length());
        }

        for (ArchiveData ad : mapArchivDaten.values()) {
            Data datenFs = ad.getData();

            if (datenFs != null) {
                long wert = datenFs.getUnscaledValue(attributName).intValue();

                if (wert >= -100000) {
                    ProzentWert += wert;
                    anzVerfuegbareDaten++;
                }
            }
        }

        if (anzVerfuegbareDaten == 0) {
            ProzentWert = -100001;
        } else {
            ProzentWert = Math.round((float) ProzentWert / (float) anzVerfuegbareDaten);
        }

        return ProzentWert;
    }

    /**
     * Berechnet einen gemittelten Q-Wert aus zwei Werten
     *
     * @param wert1 Wert 1
     * @param wert2 Wert 2
     *
     * @return Gemittelteter Q-Wert aus zwei Werten.
     */
    private long aggregiereQWert(long wert1, long wert2) {

        // Beide Werte sind gültig, dann einfach die Summer zurückgeben
        if ((wert1 >= 0) && (wert2 >= 0)) {
            return wert1 + wert2;
        }

        // Nur ein Wert gültig, dann diesen Wert zurückgeben
        if (wert1 >= 0) {
            return wert1;
        }

        if (wert2 >= 0) {
            return wert2;
        }

        // Keine gültigen Werte --> "nicht ermittelbar" zurückgeben
        return -1;
    }

    /**
     * Berechnet die Q-Werte aus einer Liste von Archivdatensätzen für einen Zeitraum (Tag, Monat, Quartal oder Jahr)
     *
     * @param mapArchivDaten       Map mit den zeitlich sortierten Archivdatensätzen für einen Zeitraum.
     * @param attributName         Attributname am MQ. Bei FS wird umgerechnet, indem erster Buchstabe klein geschrieben
     *                             wird
     * @param isMqWert             <code>true</code>, wenn Archivdatensätze aus der MQ ATG bestehen, <code>false</code>,
     *                             wenn Archivdatensätze aus den FS ATG bestehen (Attributnamen beginnen in diesem Fall
     *                             mit einem Kleinbuchstaben).
     * @param anzahlIntervalleSoll Anzahl der Intervalle, die im !fehlerfreien! Fall vorhanden sein müssen.
     *
     * @return Q-Werte der Klassen aus den Archivdaten des Zeitraums für das angegebene Attribut.
     */
    private long aggregiereQWertAusArchivDaten(Map<Long, ArchiveData> mapArchivDaten, String attributName, boolean isMqWert, int anzahlIntervalleSoll) {
        long QWert               = 0;
        int  anzVerfuegbareDaten = 0;
        int  anzFehlendeDaten;

        // Attributname anpassen, erster Buchstabe muss klein geschrieben werden, wenn es keine MQ-ATG-Daten sind
        if (!isMqWert) {
            attributName = attributName.substring(0, 1).toLowerCase() + attributName.substring(1, attributName.length());
        }

        for (ArchiveData ad : mapArchivDaten.values()) {
            Data datenFs = ad.getData();

            if (datenFs != null) {
                long wert = datenFs.getUnscaledValue(attributName).longValue();

                if (wert >= 0) {
                    QWert += wert;
                    anzVerfuegbareDaten++;
                }
            }
        }

        if (anzVerfuegbareDaten == 0) {
            QWert = -1;
        } else {
            anzFehlendeDaten = anzahlIntervalleSoll - anzVerfuegbareDaten;

            // Kann eigentlich nicht vorkommen, es müssten mehr Intervalldaten als Intervalle vorliegen
            if (anzFehlendeDaten < 0) {
                StringBuilder meldungsText = new StringBuilder();
                int           i            = 0;

                for (long zeit : mapArchivDaten.keySet()) {
                    meldungsText.append(String.format("%2d:%s%n", i++, datumsFormatierer.format(new Date(zeit))));
                }

                debug.warning(String.format("Anzahl fehlende Daten bei der Intervallberechung ist kleiner als 0 (AnzIntervallSoll %d - AnzVerfuegbareDaten %d = %d)! %nDarf eigentlich nicht vorkommen, es müssten mehr Intervalldaten als Intervalle vorliegen:%n%s ", anzahlIntervalleSoll, anzVerfuegbareDaten, anzFehlendeDaten, meldungsText.toString()));
                anzFehlendeDaten = 0;
            }

            QWert = Math.round((float) QWert * (1.0 + ((float) anzFehlendeDaten / (float) anzVerfuegbareDaten)));

            // Für DTV Monat, Quartal und Jahr soll nicht die Gesamtsumme
            // sondern die durchschnittlich tägliche Verkehrsstärke berechnet werden,
            // d.h. der errechnete Wert muss noch durch die Anzahl der Soll-Intervalle geteilt werden
            if (isMqWert) {
                QWert = Math.round((float) QWert / (float) anzahlIntervalleSoll);
            }
        }

        return QWert;
    }

    /**
     * Berechnet die Q-Werte aus einer Liste von Archivdatensätzen für einen Tag
     *
     * @param mapArchivDaten       Map mit den zeitlich sortierten Archivdatensätzen für einen Zeitraum.
     * @param attributName         Attributname am MQ in der atg.verkehrsDatenLangzeitIntervallGeschwindigkeitVerteilungFS.
     * @param anzahlIntervalleSoll Anzahl der Intervalle, die im !fehlerfreien! Fall vorhanden sein müssen.
     *
     * @return Q-Werte der Klassen aus den Archivdaten des Zeitraums für das angegebene Attribut.
     */
    private long aggregiereQWertAusArchivDatenV2004(Map<Long, ArchiveData> mapArchivDaten, String attributName, int anzahlIntervalleSoll) {
        long QWert               = 0;
        int  anzVerfuegbareDaten = 0;
        int  anzFehlendeDaten;

        // Abbildung der Namen V2007 in der atg.verkehrsDatenLangzeitIntervallGeschwindigkeitVerteilungFS
        // auf V2004 in der atg.verkehrsDatenLangzeitIntervall
        Map<String, String> mapNameV2007AufV2004 = new HashMap<String, String>();

        mapNameV2007AufV2004.put("QKfzNk", "qKfzNk");
        mapNameV2007AufV2004.put("QKrad", "qKrad");
        mapNameV2007AufV2004.put("QPkw", "qPkw");
        mapNameV2007AufV2004.put("QLfw", "qLfw");
        mapNameV2007AufV2004.put("QPkwA", "qPkwA");
        mapNameV2007AufV2004.put("QLkw", "qLkw");
        mapNameV2007AufV2004.put("QLkwA", "qLkwA");
        mapNameV2007AufV2004.put("QSattelKfz", "qSattelKfz");
        mapNameV2007AufV2004.put("QBus", "qBus");
        mapNameV2007AufV2004.put("QSGV", "qSGV");

        // Attributname anpassen auf die Namen in atg.verkehrsDatenLangzeitIntervall
        attributName = mapNameV2007AufV2004.get(attributName);

        for (ArchiveData ad : mapArchivDaten.values()) {
            Data datenFs = ad.getData();

            if (datenFs != null) {

                // qSGV-Wert muss aus qLkw, qLkwA und qSattelKfz ermittelt werden
                if (attributName.equals("qSGV")) {
                    long qLkw       = datenFs.getItem("qLkw").getUnscaledValue("Wert").longValue();
                    long qLkwA      = datenFs.getItem("qLkwA").getUnscaledValue("Wert").longValue();
                    long qSattelKfz = datenFs.getItem("qSattelKfz").getUnscaledValue("Wert").longValue();

                    // Falls mindestens ein Teilwert gültig ist, qSGV berechnen
                    long wert = -1;

                    if ((qLkw >= 0) || (qLkwA >= 0) || (qSattelKfz >= 0)) {
                        wert = Math.max(0, qLkw) + Math.max(0, qLkwA) + Math.max(0, qSattelKfz);
                    }

                    if (wert >= 0) {
                        QWert += wert;
                        anzVerfuegbareDaten++;
                    }
                } else {
                    long wert = datenFs.getItem(attributName).getUnscaledValue("Wert").longValue();

                    if (wert >= 0) {
                        QWert += wert;
                        anzVerfuegbareDaten++;
                    }
                }
            }
        }

        if (anzVerfuegbareDaten == 0) {
            QWert = -1;
        } else {
            anzFehlendeDaten = anzahlIntervalleSoll - anzVerfuegbareDaten;

            // Kann eigentlich nicht vorkommen, es müssten mehr Intervalldaten als Intervalle vorliegen
            if (anzFehlendeDaten < 0) {
                StringBuilder meldungsText = new StringBuilder();
                int           i            = 0;

                for (long zeit : mapArchivDaten.keySet()) {
                    meldungsText.append(String.format("%2d:%s%n", i++, datumsFormatierer.format(new Date(zeit))));
                }

                debug.warning(String.format("Anzahl fehlende Daten bei der Intervallberechung ist kleiner als 0 (AnzIntervallSoll %d - AnzVerfuegbareDaten %d = %d)! %nDarf eigentlich nicht vorkommen, es müssten mehr Intervalldaten als Intervalle vorliegen:%n%s ", anzahlIntervalleSoll, anzVerfuegbareDaten, anzFehlendeDaten, meldungsText.toString()));
                anzFehlendeDaten = 0;
            }

            QWert = Math.round((float) QWert * (1.0 + ((float) anzFehlendeDaten / (float) anzVerfuegbareDaten)));
        }

        return QWert;
    }

    /**
     * Berechnet die Q-Werte für Klassen aus einer Liste von Archivdatensätzen für einen Zeitraum (Tag, Monat, Quartal
     * oder Jahr)
     *
     * @param mapArchivDaten       Map mit den zeitlich sortierten Archivdatensätzen für einen Zeitraum.
     * @param attributName         Attributname am MQ. Bei FS wird umgerechnet, indem erster Buchstabe klein geschrieben
     *                             wird
     * @param isMqWert             <code>true</code>, wenn Archivdatensätze aus der MQ ATG bestehen, <code>false</code>,
     *                             wenn Archivdatensätze aus den FS ATG bestehen (Attributnamen beginnen in diesem Fall
     *                             mit einem Kleinbuchstaben).
     * @param anzahlIntervalleSoll Anzahl der Intervalle, die im !fehlerfreien! Fall vorhanden sein müssen.
     *
     * @return Q-Werte der Klassen aus den Archivdaten des Zeitraums für das angegebene Attribut.
     */
    private long[] aggregiereQWertKlassenAusArchivDaten(Map<Long, ArchiveData> mapArchivDaten, String attributName, boolean isMqWert, int anzahlIntervalleSoll) {

        // Anzahl der Klassen abhängig vom Parameter ermitteln
        Parameter p          = Parameter.getInstanz();
        int       anzKlassen = 0;

        if (attributName.equals("QLVoGeschwKlasse")) {
            anzKlassen = p.getGrenzenLVO().length + 1;
        }

        if (attributName.equals("QSGVGeschwKlasse")) {
            anzKlassen = p.getGrenzenSGV().length + 1;
        }

        if (attributName.equals("QBPAGeschwKlasse")) {
            anzKlassen = p.getGrenzenBPA().length + 1;
        }

        long  QWert[]             = new long[anzKlassen];
        int[] anzVerfuegbareDaten = new int[anzKlassen];
        int[] anzFehlendeDaten    = new int[anzKlassen];

        // Attributname anpassen, erster Buchstabe muss klein geschrieben werden, wenn es keine MQ-ATG-Daten sind
        if (!isMqWert) {
            attributName = attributName.substring(0, 1).toLowerCase() + attributName.substring(1, attributName.length());
        }

        for (ArchiveData ad : mapArchivDaten.values()) {
            Data datenFs = ad.getData();

            if (datenFs != null) {
                Data.Array da = datenFs.getArray(attributName);

                for (int i = 0; i < da.getLength(); i++) {
                    long wert = da.getUnscaledValue(i).longValue();

                    if (wert >= 0) {
                        QWert[i] += wert;
                        anzVerfuegbareDaten[i]++;
                    }
                }
            }
        }

        for (int i = 0; i < QWert.length; i++) {
            if (anzVerfuegbareDaten[i] == 0) {
                QWert[i] = -1;
            } else {
                anzFehlendeDaten[i] = anzahlIntervalleSoll - anzVerfuegbareDaten[i];

                // Kann eigentlich nicht vorkommen, es müssten mehr Intervalldaten als Intervalle vorliegen
                if (anzFehlendeDaten[i] < 0) {
                    debug.warning(String.format("Anzahl fehlende Daten bei der Intervallberechung ist kleiner als 0 (AnzIntervallSoll %d - AnzVerfuegbareDaten %d = %d)! %nDarf eigentlich nicht vorkommen, es müssten mehr Intervalldaten als Intervalle vorliegen ", anzahlIntervalleSoll, anzVerfuegbareDaten[i], anzFehlendeDaten[i]));
                    anzFehlendeDaten[i] = 0;
                }

                QWert[i] = Math.round((float) QWert[i] * (1.0 + ((float) anzFehlendeDaten[i] / (float) anzVerfuegbareDaten[i])));

                // Für DTV Monat, Quartal und Jahr soll nicht die Gesamtsumme
                // sondern die durchschnittlich tägliche Verkehrsstärke berechnet werden,
                // d.h. der errechnete Wert muss noch durch die Anzahl der Soll-Intervalle geteilt werden
                if (isMqWert) {
                    QWert[i] = Math.round((float) QWert[i] / (float) anzahlIntervalleSoll);
                }
            }
        }

        return QWert;
    }

    /**
     * Berechnet die V-Werte aus einer Liste von Archivdatensätzen für einen Zeitraum (Tag, Monat, Quartal oder Jahr)
     *
     * @param mapArchivDaten Map mit den zeitlich sortierten Archivdatensätzen für einen Zeitraum.
     * @param attributName   Attributname am MQ. Bei FS wird umgerechnet, indem erster Buchstabe klein geschrieben wird
     * @param isMqWert       <code>true</code>, wenn Archivdatensätze aus der MQ ATG bestehen, <code>false</code>, wenn
     *                       Archivdatensätze aus den FS ATG bestehen (Attributnamen beginnen in diesem Fall mit einem
     *                       Kleinbuchstaben).
     *
     * @return V-Werte der Klassen aus den Archivdaten des Zeitraums für das angegebene Attribut.
     */
    private int aggregiereVWertAusArchivDaten(Map<Long, ArchiveData> mapArchivDaten, String attributName, boolean isMqWert) {
        int VWert               = 0;
        int anzVerfuegbareDaten = 0;

        // Attributname anpassen, erster Buchstabe muss klein geschrieben werden, wenn es keine MQ-ATG-Daten sind
        if (!isMqWert) {
            attributName = attributName.substring(0, 1).toLowerCase() + attributName.substring(1, attributName.length());
        }

        for (ArchiveData ad : mapArchivDaten.values()) {
            Data datenFs = ad.getData();

            if (datenFs != null) {
                int wert = datenFs.getUnscaledValue(attributName).intValue();

                if (wert >= 0) {
                    VWert += wert;
                    anzVerfuegbareDaten++;
                }
            }
        }

        if (anzVerfuegbareDaten == 0) {
            VWert = -1;
        } else {
            VWert = Math.round((float) VWert / (float) anzVerfuegbareDaten);
        }

        return VWert;
    }

    /**
     * Berechnet einen gemittelten V-Wert aus zwei Werten
     *
     * @param wert1 Wert 1
     * @param wert2 Wert 2
     *
     * @return Gemittelteter V-Wert aus zwei Werten.
     */
    private int aggregiereVWerte(int wert1, int wert2) {

        // Beide Werte sind gültig, dann einfach den Mittelwert zurückgeben
        if ((wert1 >= 0) && (wert2 >= 0)) {
            return (int) Math.round((float) (wert1 + wert2) / 2.0);
        }

        // Nur ein Wert gültig, dann diesen Wert zurückgeben
        if (wert1 >= 0) {
            return wert1;
        }

        if (wert2 >= 0) {
            return wert2;
        }

        // Keine gültigen Werte --> "nicht ermittelbar" zurückgeben
        return -1;
    }

    /**
     * Methode, die den eigentlichen Auftrag ausführt.
     *
     * @param auftrag Der auszuführende Auftrag.
     *
     * @return Meldung, die den Ablauf der Ausführung des Auftrags enthält. Wird für die Statusdarstellung verwendet.
     */
    public String ausfuehren(Auftrag auftrag) {
        switch (auftrag.getIntervallDauer()) {
        case DTV_TAG :
            return ausfuehrenDTVTag(auftrag);

        // return auftrag.toString();
        case DTV_MONAT :
        case DTV_QUARTAL :
        case DTV_JAHR :
            return ausfuehrenDTVMonatOderQuartalOderJahr(auftrag);

        case MSV_JAHR :
            return ausfuehrenMSVJahr(auftrag);

        default :
            return String.format("Ungültiger Auftrag (kein gültiger Auftragstyp/IntervallDauer): %s", auftrag);
        }
    }

    /**
     * Berechnet auf Basis der Auftragsdaten die Monats, Quartals oder Jahreswerte für die beiden MQ und die LZZS. Die
     * Methode ermittelt die Archivdaten für das Intervall und aggregiert diese zur gewünschten Stufe hoch.
     *
     * @param auftrag Auftragsdaten, auf desen Basis die Aggregation erfolgt.
     *
     * @return Meldung über den Verlauf bzw. das Ergebnis der Aggregation zur Puplikation in den DaV.
     */
    private String ausfuehrenDTVMonatOderQuartalOderJahr(Auftrag auftrag) {

        // Meldungstext für die Auftragsabwicklung (wird über DaV als Status ausgegeben)
        StringBuilder meldung = new StringBuilder();

        // String für Teilmeldungen
        String teilMeldung;

        meldung.append(auftrag.toString());

        ArchiveDataSpecification anfrage = null;         // Anfrage an das Archiv
        ArchiveDataQueryResult   archivAnfrageErgebnis;  // Ergebnisdatensatz der Archivanfrage
        ArchiveDataStream        archivStream[];         // ArchivStream für den Zugriff auf die Archivantwort
        ArchiveData              ad;                     // Archivdaten

        // Archivdaten abfragen
        ArchiveTimeSpecification   zeitSpezifikation = new ArchiveTimeSpecification(TimingType.DATA_TIME, false, auftrag.getIntervallBeginn(), auftrag.getIntervallEnde() - 1);
        ArchiveDataKindCombination archivDatenTypen  = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE, ArchiveDataKind.ONLINE_DELAYED);
        DataDescription            datenBeschreibung;

        // Aspekt abhängig vom Auftragstyp (IntervallDauer) setzen
        // Der Aspekt wird auf den Wert gesetzt, den man zur Aggregation des Auftrags benötigt,
        // also bei Auftrag Monat --> Aspekt Tag etc.
        Aspect asp = null;

        switch (auftrag.getIntervallDauer()) {
        case DTV_MONAT :
            asp = _aspDTVTag;

            break;

        case DTV_QUARTAL :
            asp = _aspDTVMonat;

            break;

        case DTV_JAHR :
            asp = _aspDTVQuartal;

            break;
        }

        datenBeschreibung = new DataDescription(_atgMqDTV, asp);

        List<ResultData> listeErgebnisDatenMq = new ArrayList<ResultData>();

        // Schleife über MQ einer LZZS
        try {
            for (Mq mq : _listeMq) {

                // Nach Zeitstempel sortierte Map mit den Archivdaten eines MQ für
                // - Monat (auf Tagesbasis)
                // - Quartal (auf Monatsbasis)
                // - Jahr (auf Quartalsbasis)
                Map<Long, ArchiveData> mapArchivIntervallDaten = new TreeMap<Long, ArchiveData>();

                try {
                    anfrage               = new ArchiveDataSpecification(zeitSpezifikation, archivDatenTypen, ArchiveOrder.BY_INDEX, ArchiveRequestOption.NORMAL, datenBeschreibung, mq.getMq());
                    archivAnfrageErgebnis = _archiv.request(ArchiveQueryPriority.LOW, anfrage);
                    archivStream          = archivAnfrageErgebnis.getStreams();

                    // Alle Daten einsortieren nach Datenzeitstempel. Dadurch werden bei doppelten Datensätzen immer nur die neusten berücksichtigt.
                    while ((ad = archivStream[0].take()) != null) {
                        if (ad.getDataType().equals(DataState.DATA)) {

                            // Jetzt noch Prüfen, ob Zeitstempel innerhalb der angefragen Zeitraums liegt (Problem, wenn Archivdaten zeitlich vor erstem Start des Archivs liegen)
                            if ((auftrag.getIntervallBeginn() <= ad.getDataTime()) && (ad.getDataTime() < auftrag.getIntervallEnde())) {
                                mapArchivIntervallDaten.put(ad.getDataTime(), ad);
                                debug.fine(String.format("%s [%s] (%d) %s-%s [%s] : %s%n", ad.getDataType(), ad.getDataKind(), ad.getDataIndex(), datumsFormatierer.format(new Date(ad.getDataTime())), datumsFormatierer.format(new Date(ad.getArchiveTime())), ad.getObject(), ad.getData()));
                            }
                        } else {
                            debug.fine(String.format("%s : %s%n", ad.getDataType(), datumsFormatierer.format(new Date(ad.getDataTime()))));
                        }
                    }
                }
                catch (InterruptedException e) {
                    teilMeldung = String.format("%nArchivantwort auf Archivanfrage [%s] konnte nicht nicht vollständig ermittelt werden (unerwartete Unterbrechung bei der Übertragung der Antwort).", anfrage);
                    meldung.append(teilMeldung);
                    debug.warning(teilMeldung, e);
                }
                catch (IllegalStateException e) {
                    teilMeldung = String.format("Archivantwort auf Archivanfrage [%s] konnte nicht ermittelt werden (Keine Ergebnisdaten ermittelbar).", anfrage);
                    meldung.append(teilMeldung);
                    debug.warning(teilMeldung, e);
                }
                catch (IOException e) {
                    teilMeldung = String.format("Archivantwort auf Archivanfrage [%s] konnte nicht vollständig ermittelt werden (Probleme in der Kommunikation mit dem Archivsystem aufgetreten. Nicht alle Datensätze wurden übertragen).", anfrage);
                    meldung.append(teilMeldung);
                    debug.warning(teilMeldung, e);
                }

                listeErgebnisDatenMq.add(berechneDTVMonatOderQuartalOderJahrMq(auftrag, mq, mapArchivIntervallDaten));
            }
        }
        catch (Exception e) {
            teilMeldung = String.format("Berechnung DTVMonatOderQuartalOderJahrMq für LZZS [%s] für Ergebnisdaten LZZS und MQ t.w. fehlgeschlagen. Statistikdaten für LZZS wurde nicht berechnet.", _lzzs.getPid());
            meldung.append(teilMeldung);
            debug.warning(teilMeldung, e);

            // Meldung mit dem Ergebnis für die Debugausgabe bzw. für die Fortschrittsanzeige zurückgeben.
            return meldung.toString();
        }

        // Archivdaten für das Intervall aggregrieren
        ResultData ergebnisDatenLzzs = berechneDTVLzzs(auftrag, listeErgebnisDatenMq);

        // Aggregierte Daten an DaV senden
        try {

            // Es gibt IMMER GENAU zwei Mq (wird durch DatKat gewährleistet).
            _dav.sendData(listeErgebnisDatenMq.get(0));
            _dav.sendData(listeErgebnisDatenMq.get(1));
            _dav.sendData(ergebnisDatenLzzs);
        }
        catch (SendSubscriptionNotConfirmed e) {
            teilMeldung = String.format("Sendeanmeldung für LZZS [%s] für Ergebnisdaten LZZS und MQ t.w. noch nicht erfolgt. Statistikdaten konnten nicht an den Datenverteiler gesendet werden.", _lzzs.getPid());
            meldung.append(teilMeldung);
            debug.warning(teilMeldung, e);
        }

        // Meldung mit dem Ergebnis für die Debugausgabe bzw. für die Fortschrittsanzeige zurückgeben.
        return meldung.toString();
    }

    /**
     * Berechnet auf Basis der Auftragsdaten die Tagesdaten für die beiden MQ und die LZZS. Zur Berechnung der MQ-Werte
     * werden hier die FS-Werte an den MQ verwendet. Abhängig vom Aufrufparameter <code>-aggregationAusErfasstenDaten</code>
     * werden dabei entweder die erfassten Daten am FS verwendet oder die aus dem BASt-Band Import vorliegenden Daten.
     * Weiterhin wird geprüft, ob vor der Aggregation die 8+1-q-Werte aus aus den reimportierten BASt V2004-Werten
     * (atg.verkehrsDatenLangZeitIntervall, asp.messWertErsetzung) übernommen werden sollen
     * <code>_isQWerteAusReimportAtgVerkehrsDatenLangzeitIntervall==true</code>. Dies erfolgt aber nur, wenn auch
     * <code>isAggregationAusErfasstenDaten==true</code> ist.
     * <p/>
     * Die Methode ermittelt die Archivdaten für das Intervall und aggregiert diese zur gewünschten Stufe hoch.
     *
     * @param auftrag Auftragsdaten, auf desen Basis die Aggregation erfolgt.
     *
     * @return Meldung über den Verlauf bzw. das Ergebnis der Aggregation zur Puplikation in den DaV.
     */
    private String ausfuehrenDTVTag(Auftrag auftrag) {

        // Meldungstext für die Auftragsabwicklung (wird über DaV als Status ausgegeben)
        StringBuilder meldung = new StringBuilder();

        // String für Teilmeldungen
        String teilMeldung;

        meldung.append(auftrag.toString());

        ArchiveDataSpecification anfrage = null;         // Anfrage an das Archiv
        ArchiveDataQueryResult   archivAnfrageErgebnis;  // Ergebnisdatensatz der Archivanfrage
        ArchiveDataStream        archivStream[];         // ArchivStream für den Zugriff auf die Archivantwort
        ArchiveData              ad;                     // Archivdaten

        // Archivdaten abfragen
        ArchiveTimeSpecification   zeitSpezifikation = new ArchiveTimeSpecification(TimingType.DATA_TIME, false, auftrag.getIntervallBeginn(), auftrag.getIntervallEnde() - 1);
        ArchiveDataKindCombination archivDatenTypen  = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE, ArchiveDataKind.ONLINE_DELAYED);
        DataDescription            datenBeschreibung, datenBeschreibungV2004;

        datenBeschreibungV2004 = new DataDescription(_atgFSVerkehrsDatenLangZeitIntervall, _aspMessWertErsetzung);

        if (_isAggregationAusErfasstenDaten) {
            datenBeschreibung = new DataDescription(_atgFsDTV, _aspExterneErfassung);
        } else {
            datenBeschreibung = new DataDescription(_atgFsDTV, _aspMessWertErsetzung);
        }

        List<ResultData> listeErgebnisDatenMq = new ArrayList<ResultData>();

        // Schleife über MQ einer LZZS
        for (Mq mq : _listeMq) {

            // Liste mit den Maps der aktuellen Datensätze für das zu berücksichtigende Intervall (Schlüssel = Datenzeitstempel, Wert = Archivdatensatz)
            List<Map<Long, ArchiveData>> listeMapsArchivIntervallDatenFsFuerMqV2004 = new ArrayList<Map<Long, ArchiveData>>();
            List<Map<Long, ArchiveData>> listeMapsArchivIntervallDatenFsFuerMq      = new ArrayList<Map<Long, ArchiveData>>();

            // Schleife über die FS eines MQ
            try {

                // FS-Werte am MQ einlesen (für reimportierte BASt-V2004 Daten)
                if (!_isAggregationAusErfasstenDaten && _isQWerteAusReimportAtgVerkehrsDatenLangzeitIntervall) {
                    for (Mq.Fs fs : mq.getListeFs()) {
                        try {

                            // Nach Zeitstempel sortierte Map mit den Archivdaten eines FS für einen Tag
                            Map<Long, ArchiveData> mapArchivIntervallDatenImportV2004 = new TreeMap<Long, ArchiveData>();

                            anfrage               = new ArchiveDataSpecification(zeitSpezifikation, archivDatenTypen, ArchiveOrder.BY_INDEX, ArchiveRequestOption.NORMAL, datenBeschreibungV2004, fs.getFs());
                            archivAnfrageErgebnis = _archiv.request(ArchiveQueryPriority.LOW, anfrage);
                            archivStream          = archivAnfrageErgebnis.getStreams();

                            // Alle Daten einsortieren nach Datenzeitstempel. Dadurch werden bei doppelten Datensätzen immer nur die neusten berücksichtigt.
                            while ((ad = archivStream[0].take()) != null) {
                                if (ad.getDataType().equals(DataState.DATA)) {

                                    // Jetzt noch Prüfen, ob Zeitstempel innerhalb der angefragen Zeitraums liegt (Problem, wenn Archivdaten zeitlich vor erstem Start des Archivs liegen)
                                    if ((auftrag.getIntervallBeginn() <= ad.getDataTime()) && (ad.getDataTime() < auftrag.getIntervallEnde())) {
                                        mapArchivIntervallDatenImportV2004.put(ad.getDataTime(), ad);
                                        debug.fine(String.format("%s [%s] (%d) %s-%s [%s] : %s%n", ad.getDataType(), ad.getDataKind(), ad.getDataIndex(), datumsFormatierer.format(new Date(ad.getDataTime())), datumsFormatierer.format(new Date(ad.getArchiveTime())), ad.getObject(), ad.getData()));
                                    }
                                } else {
                                    debug.fine(String.format("%s : %s%n", ad.getDataType(), datumsFormatierer.format(new Date(ad.getDataTime()))));
                                }
                            }

                            listeMapsArchivIntervallDatenFsFuerMqV2004.add(mapArchivIntervallDatenImportV2004);
                        }
                        catch (InterruptedException e) {
                            teilMeldung = String.format("%nArchivantwort auf Archivanfrage [%s] konnte nicht nicht vollständig ermittelt werden (unerwartete Unterbrechung bei der Übertragung der Antwort).", anfrage);
                            meldung.append(teilMeldung);
                            debug.warning(teilMeldung, e);
                        }
                        catch (IllegalStateException e) {
                            teilMeldung = String.format("Archivantwort auf Archivanfrage [%s] konnte nicht ermittelt werden (Keine Ergebnisdaten ermittelbar).", anfrage);
                            meldung.append(teilMeldung);
                            debug.warning(teilMeldung, e);
                        }
                        catch (IOException e) {
                            teilMeldung = String.format("Archivantwort auf Archivanfrage [%s] konnte nicht vollständig ermittelt werden (Probleme in der Kommunikation mit dem Archivsystem aufgetreten. Nicht alle Datensätze wurden übertragen).", anfrage);
                            meldung.append(teilMeldung);
                            debug.warning(teilMeldung, e);
                        }
                    }
                }

                // FS-Werte am MQ einlesen (für BASt-V2007 Daten)
                for (Mq.Fs fs : mq.getListeFs()) {
                    try {

                        // Nach Zeitstempel sortierte Map mit den Archivdaten eines FS für einen Tag
                        Map<Long, ArchiveData> mapArchivIntervallDaten = new TreeMap<Long, ArchiveData>();

                        anfrage               = new ArchiveDataSpecification(zeitSpezifikation, archivDatenTypen, ArchiveOrder.BY_INDEX, ArchiveRequestOption.NORMAL, datenBeschreibung, fs.getFs());
                        archivAnfrageErgebnis = _archiv.request(ArchiveQueryPriority.LOW, anfrage);
                        archivStream          = archivAnfrageErgebnis.getStreams();

                        // Alle Daten einsortieren nach Datenzeitstempel. Dadurch werden bei doppelten Datensätzen immer nur die neusten berücksichtigt.
                        while ((ad = archivStream[0].take()) != null) {
                            if (ad.getDataType().equals(DataState.DATA)) {

                                // Jetzt noch Prüfen, ob Zeitstempel innerhalb der angefragen Zeitraums liegt (Problem, wenn Archivdaten zeitlich vor erstem Start des Archivs liegen)
                                if ((auftrag.getIntervallBeginn() <= ad.getDataTime()) && (ad.getDataTime() < auftrag.getIntervallEnde())) {
                                    mapArchivIntervallDaten.put(ad.getDataTime(), ad);
                                    debug.fine(String.format("%s [%s] (%d) %s-%s [%s] : %s%n", ad.getDataType(), ad.getDataKind(), ad.getDataIndex(), datumsFormatierer.format(new Date(ad.getDataTime())), datumsFormatierer.format(new Date(ad.getArchiveTime())), ad.getObject(), ad.getData()));
                                }
                            } else {
                                debug.fine(String.format("%s : %s%n", ad.getDataType(), datumsFormatierer.format(new Date(ad.getDataTime()))));
                            }
                        }

                        listeMapsArchivIntervallDatenFsFuerMq.add(mapArchivIntervallDaten);
                    }
                    catch (InterruptedException e) {
                        teilMeldung = String.format("%nArchivantwort auf Archivanfrage [%s] konnte nicht nicht vollständig ermittelt werden (unerwartete Unterbrechung bei der Übertragung der Antwort).", anfrage);
                        meldung.append(teilMeldung);
                        debug.warning(teilMeldung, e);
                    }
                    catch (IllegalStateException e) {
                        teilMeldung = String.format("Archivantwort auf Archivanfrage [%s] konnte nicht ermittelt werden (Keine Ergebnisdaten ermittelbar).", anfrage);
                        meldung.append(teilMeldung);
                        debug.warning(teilMeldung, e);
                    }
                    catch (IOException e) {
                        teilMeldung = String.format("Archivantwort auf Archivanfrage [%s] konnte nicht vollständig ermittelt werden (Probleme in der Kommunikation mit dem Archivsystem aufgetreten. Nicht alle Datensätze wurden übertragen).", anfrage);
                        meldung.append(teilMeldung);
                        debug.warning(teilMeldung, e);
                    }
                }

                listeErgebnisDatenMq.add(berechneDTVTagMq(auftrag, mq, listeMapsArchivIntervallDatenFsFuerMq, listeMapsArchivIntervallDatenFsFuerMqV2004));
            }
            catch (Exception e) {
                teilMeldung = String.format("Ausführung der Berechnung von DTVTag für MQ [%s] t.w. fehlgeschlagen. Statistikdaten für MQ wurde nicht berechnet.", mq.getMq().getPid());
                meldung.append(teilMeldung);
                debug.warning(teilMeldung, e);

                // Meldung mit dem Ergebnis für die Debugausgabe bzw. für die Fortschrittsanzeige zurückgeben.
                return meldung.toString();
            }
        }

        // Archivdaten für das Intervall aggregrieren
        ResultData ergebnisDatenLzzs = berechneDTVLzzs(auftrag, listeErgebnisDatenMq);

        // Aggregierte Daten an DaV senden
        try {

            // Es gibt IMMER GENAU zwei Mq (wird durch DatKat gewährleistet).
            _dav.sendData(listeErgebnisDatenMq.get(0));
            _dav.sendData(listeErgebnisDatenMq.get(1));
            _dav.sendData(ergebnisDatenLzzs);
        }
        catch (SendSubscriptionNotConfirmed e) {
            teilMeldung = String.format("Sendeanmeldung für LZZS [%s] für Ergebnisdaten LZZS und MQ noch nicht erfolgt. Statistikdaten konnten nicht an den Datenverteiler gesendet werden.", _lzzs.getPid());
            meldung.append(teilMeldung);
            debug.warning(teilMeldung, e);
        }

        // Meldung mit dem Ergebnis für die Debugausgabe bzw. für die Fortschrittsanzeige zurückgeben.
        return meldung.toString();
    }

    private String ausfuehrenMSVJahr(Auftrag auftrag) {

        // Meldungstext für die Auftragsabwicklung (wird über DaV als Status ausgegeben)
        StringBuilder meldung = new StringBuilder();

        // String für Teilmeldungen
        String teilMeldung;

        meldung.append(auftrag.toString());

        ArchiveDataSpecification anfrage = null;         // Anfrage an das Archiv
        ArchiveDataQueryResult   archivAnfrageErgebnis;  // Ergebnisdatensatz der Archivanfrage
        ArchiveDataStream        archivStream[];         // ArchivStream für den Zugriff auf die Archivantwort
        ArchiveData              ad;                     // Archivdaten

        // Archivdaten abfragen
        ArchiveTimeSpecification   zeitSpezifikation = new ArchiveTimeSpecification(TimingType.DATA_TIME, false, auftrag.getIntervallBeginn(), auftrag.getIntervallEnde() - 1);
        ArchiveDataKindCombination archivDatenTypen  = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE, ArchiveDataKind.ONLINE_DELAYED);
        DataDescription            datenBeschreibung;

        if (_isAggregationAusErfasstenDaten) {
            datenBeschreibung = new DataDescription(_atgFsDTV, _aspExterneErfassung);
        } else {
            datenBeschreibung = new DataDescription(_atgFsDTV, _aspMessWertErsetzung);
        }

        // Datenstruktur für jede Stunde des Jahres jeweils für die QKfzGesamt-Werte der beiden MQ und der LZZS initialisieren
        // Struktur besteht aus einer Map mit Schlüssel = Zeit und Daten = Einträgen für MQ1, MQ2 und LZZS
        final long                       stundeInMs        = 60 * 60 * 1000;  // Anzahl Millisekunden einer Stunde
        final int                        anzStundenProJahr = (int) ((auftrag.getIntervallEnde() - auftrag.getIntervallBeginn()) / stundeInMs);
        Map<Long, QKfzGesamtStundenWert> mapDaten          = new HashMap<Long, QKfzGesamtStundenWert>(anzStundenProJahr);

        for (int i = 0; i < anzStundenProJahr; i++) {
            mapDaten.put(auftrag.getIntervallBeginn() + (stundeInMs * i), new QKfzGesamtStundenWert());
        }

        // Schleife über MQ einer LZZS (sind immer genau 2)
        int indexMq = 1;

        for (Mq mq : _listeMq) {

            // Schleife über die FS eines MQ
            for (Mq.Fs fs : mq.getListeFs()) {
                try {

                    // Nach Zeitstempel sortierte Map mit den qKfzGesamt-Wert eines FS für ein Jahr
                    Map<Long, Long> mapArchivIntervallDaten = new TreeMap<Long, Long>();

                    anfrage               = new ArchiveDataSpecification(zeitSpezifikation, archivDatenTypen, ArchiveOrder.BY_INDEX, ArchiveRequestOption.NORMAL, datenBeschreibung, fs.getFs());
                    archivAnfrageErgebnis = _archiv.request(ArchiveQueryPriority.LOW, anfrage);
                    archivStream          = archivAnfrageErgebnis.getStreams();

                    // Alle Daten einsortieren nach Datenzeitstempel. Dadurch werden bei doppelten Datensätzen immer nur die neusten berücksichtigt.
                    while ((ad = archivStream[0].take()) != null) {
                        if (ad.getDataType().equals(DataState.DATA)) {

                            // Jetzt noch Prüfen, ob Zeitstempel innerhalb der angefragen Zeitraums liegt (Problem, wenn Archivdaten zeitlich vor erstem Start des Archivs liegen)
                            if ((auftrag.getIntervallBeginn() <= ad.getDataTime()) && (ad.getDataTime() < auftrag.getIntervallEnde())) {
                                long qKfzGesamt = ermittleQKfzGesamt(ad);

                                mapArchivIntervallDaten.put(ad.getDataTime(), qKfzGesamt);
                                debug.fine(String.format("%s [%s] (%d) %s-%s [%s] : %s%n", ad.getDataType(), ad.getDataKind(), ad.getDataIndex(), datumsFormatierer.format(new Date(ad.getDataTime())), datumsFormatierer.format(new Date(ad.getArchiveTime())), ad.getObject(), ad.getData()));
                            }
                        } else {
                            debug.fine(String.format("%s : %s%n", ad.getDataType(), datumsFormatierer.format(new Date(ad.getDataTime()))));
                        }
                    }

                    // Nachdem jetzt die Daten der FS für ein Jahr eingelesen wurden, diese für den MQ eintragen
                    for (long zeitPunkt : mapArchivIntervallDaten.keySet()) {
                        QKfzGesamtStundenWert qKfzGesamtStundenWert = mapDaten.get(zeitPunkt);

                        if (qKfzGesamtStundenWert != null) {
                            qKfzGesamtStundenWert.setzeMqWert(indexMq, mapArchivIntervallDaten.get(zeitPunkt));
                        }
                    }
                }
                catch (InterruptedException e) {
                    teilMeldung = String.format("%nArchivantwort auf Archivanfrage [%s] konnte nicht nicht vollständig ermittelt werden (unerwartete Unterbrechung bei der Übertragung der Antwort).", anfrage);
                    meldung.append(teilMeldung);
                    debug.warning(teilMeldung, e);
                }
                catch (IllegalStateException e) {
                    teilMeldung = String.format("Archivantwort auf Archivanfrage [%s] konnte nicht ermittelt werden (Keine Ergebnisdaten ermittelbar).", anfrage);
                    meldung.append(teilMeldung);
                    debug.warning(teilMeldung, e);
                }
                catch (IOException e) {
                    teilMeldung = String.format("Archivantwort auf Archivanfrage [%s] konnte nicht vollständig ermittelt werden (Probleme in der Kommunikation mit dem Archivsystem aufgetreten. Nicht alle Datensätze wurden übertragen).", anfrage);
                    meldung.append(teilMeldung);
                    debug.warning(teilMeldung, e);
                }
            }

            indexMq++;
        }

        // MSV-Érgbnisdatensätze für MQ1, MQ2 und LZZS ermitteln
        ResultData ergebnisDatenMq1  = berechneMSV(auftrag, _listeMq.get(0).getMq(), mapDaten, 1);
        ResultData ergebnisDatenMq2  = berechneMSV(auftrag, _listeMq.get(1).getMq(), mapDaten, 2);
        ResultData ergebnisDatenLzzs = berechneMSV(auftrag, _lzzs, mapDaten, 0);

        // Aggregierte Daten an DaV senden
        try {

            // Es gibt IMMER GENAU zwei Mq (wird durch DatKat gewährleistet).
            _dav.sendData(ergebnisDatenMq1);
            _dav.sendData(ergebnisDatenMq2);
            _dav.sendData(ergebnisDatenLzzs);
        }
        catch (SendSubscriptionNotConfirmed e) {
            teilMeldung = String.format("Sendeanmeldung für LZZS [%s] für Ergebnisdaten LZZS und MQ noch nicht erfolgt. Statistikdaten konnten nicht an den Datenverteiler gesendet werden.", _lzzs.getPid());
            meldung.append(teilMeldung);
            debug.warning(teilMeldung, e);
        }

        // Meldung mit dem Ergebnis für die Debugausgabe bzw. für die Fortschrittsanzeige zurückgeben.
        return meldung.toString();
    }

    /**
     * Methode, die aus den aggregierten MQ-Daten die LZZS-Daten des identischen Intervalls berechnet
     *
     * @param auftrag              Der Auftrag, für den die Intervalldaten berechnet werden sollen.
     * @param listeErgebnisDatenMq Liste mit den ZWEI {@link de.bsvrz.dav.daf.main.ResultData}-Datensätzen der
     *                             aggregierten MQ-Daten für das gewünschte Intervall (Tag, Quartal, Monat, Jahr). Durch
     *                             die Datenmodellierung im DatKat ist sichergestellt, dass eine LZZS immer genau zwei
     *                             MQ hat. Die Methode setzt zudem voraus, dass in beiden Datensätzen gültige Daten
     *                             vorhanden sind.
     *
     * @return Der Ergebnisdatensatz mit den berechneten Intervalldaten.
     */
    private ResultData berechneDTVLzzs(Auftrag auftrag, List<ResultData> listeErgebnisDatenMq) {
        Data       datenLzzs = _dav.createData(_atgMqDTV);            // Ergebnissdatensatz der LZZS
        Data.Array daLzzs;                                            // Hilfskonstruktion zum Zugriff auf die Arrays mit den Klassenwerten an der LZZS
        Data.Array daMq1;                                             // Hilfskonstruktion zum Zugriff auf die Arrays mit den Klassenwerten am MQ1
        Data.Array daMq2;                                             // Hilfskonstruktion zum Zugriff auf die Arrays mit den Klassenwerten am MQ2
        Parameter  p = Parameter.getInstanz();                        // Instanz mit den Klassenparametern
        int        anz, i;                                            // Hilfsvariablen
        Aspect     asp      = null;                                   // Aspekt, mit dem Daten geschrieben werden
        Data       datenMq1 = listeErgebnisDatenMq.get(0).getData();  // Daten vom MQ1
        Data       datenMq2 = listeErgebnisDatenMq.get(1).getData();  // Daten vom MQ2

        // Aspekt auf Basis der Intervalldauer setzen
        switch (auftrag.getIntervallDauer()) {
        case DTV_TAG :
            asp = _aspDTVTag;

            break;

        case DTV_MONAT :
            asp = _aspDTVMonat;

            break;

        case DTV_QUARTAL :
            asp = _aspDTVQuartal;

            break;

        case DTV_JAHR :
            asp = _aspDTVJahr;

            break;
        }

        DataDescription datenBeschreibung = new DataDescription(_atgMqDTV, asp);

        // -----------------------------------------------------------------------------------------------
        // Initialisierung der Daten
        // -----------------------------------------------------------------------------------------------
        String[] attributNamenQWerte        = { "QKfzNk", "QKrad", "QPkw", "QLfw", "QPkwA", "QLkw", "QLkwA", "QSattelKfz", "QBus", "QSGV" };
        String[] attributNamenProzentWerte  = { "AnteilSGV" };
        String[] attributNamenVWerte        = { "VLVo", "VSGV", "VBPA", "SLVo", "SSGV", "SBPA", "V85LVo", "V85SGV", "V85BPA", "V15LVo", "V15SGV", "V15BPA" };
        String[] attributNamenQWerteKlassen = { "QLVoGeschwKlasse", "QSGVGeschwKlasse", "QBPAGeschwKlasse" };

        // QLVoGeschwKlasse vorbelegen
        anz = p.getGrenzenLVO().length + 1;
        datenLzzs.getArray("QLVoGeschwKlasse").setLength(anz);

        // QSGVGeschwKlasse vorbelegen
        anz = p.getGrenzenSGV().length + 1;
        datenLzzs.getArray("QSGVGeschwKlasse").setLength(anz);

        // QBPAGeschwKlasse vorbelegen
        anz = p.getGrenzenBPA().length + 1;
        datenLzzs.getArray("QBPAGeschwKlasse").setLength(anz);

        // VGrenzLVo vorbelegen (wird fix aus den Paramtern vorbelegt, da Grenzen beim Ergebnisdatensatz vorhanden sind.
        anz    = p.getGrenzenLVO().length;
        daLzzs = datenLzzs.getArray("VGrenzLVo");
        daLzzs.setLength(anz);

        for (i = 0; i < anz; i++) {
            daLzzs.getUnscaledValue(i).set(p.getGrenzenLVO()[i]);
        }

        // VGrenzSGV vorbelegen (wird fix aus den Paramtern vorbelegt, da Grenzen beim Ergebnisdatensatz vorhanden sind.
        anz    = p.getGrenzenSGV().length;
        daLzzs = datenLzzs.getArray("VGrenzSGV");
        daLzzs.setLength(anz);

        for (i = 0; i < anz; i++) {
            daLzzs.getUnscaledValue(i).set(p.getGrenzenSGV()[i]);
        }

        // VGrenzBPA vorbelegen (wird fix aus den Paramtern vorbelegt, da Grenzen beim Ergebnisdatensatz vorhanden sind.
        anz    = p.getGrenzenBPA().length;
        daLzzs = datenLzzs.getArray("VGrenzBPA");
        daLzzs.setLength(anz);

        for (i = 0; i < anz; i++) {
            daLzzs.getUnscaledValue(i).set(p.getGrenzenBPA()[i]);
        }

        // -----------------------------------------------------------------------------------------------
        // Berechnung des Ergebnisdatensatzes
        // -----------------------------------------------------------------------------------------------
        // Q-Werte berechnen
        for (String attributName : attributNamenQWerte) {
            datenLzzs.getUnscaledValue(attributName).set(aggregiereQWert(datenMq1.getUnscaledValue(attributName).longValue(), datenMq2.getUnscaledValue(attributName).longValue()));
        }

        // V-Werte berechnen
        for (String attributName : attributNamenVWerte) {
            datenLzzs.getUnscaledValue(attributName).set(aggregiereVWerte(datenMq1.getUnscaledValue(attributName).intValue(), datenMq2.getUnscaledValue(attributName).intValue()));
        }

        // %-Werte berechen
        for (String attributName : attributNamenProzentWerte) {
            datenLzzs.getUnscaledValue(attributName).set(aggregiereProzentWert(datenMq1.getUnscaledValue(attributName).longValue(), datenMq2.getUnscaledValue(attributName).longValue()));
        }

        // Q-Klassenwerte berechnen
        for (String attributName : attributNamenQWerteKlassen) {
            daLzzs = datenLzzs.getArray(attributName);
            daMq1  = datenMq1.getArray(attributName);
            daMq2  = datenMq2.getArray(attributName);

            for (i = 0; i < daLzzs.getLength(); i++) {
                daLzzs.getUnscaledValue(i).set(aggregiereQWert(daMq1.getUnscaledValue(i).longValue(), daMq2.getUnscaledValue(i).longValue()));
            }
        }

        // -----------------------------------------------------------------------------------------------
        // Rückgabe des Ergebnisdatensatzen
        // -----------------------------------------------------------------------------------------------
        return new ResultData(_lzzs, datenBeschreibung, auftrag.getIntervallBeginn(), datenLzzs);
    }

    /**
     * Berechnet den Ergebnisdatensatz mit den DTVMonat, DTVQuartal oder DTVJahres-Werten für einen MQ auf Basis der
     * Archivdaten der davor liegenden Aggregationsstufe.
     *
     * @param auftrag                 Auftrag (für den Tag), der bearbeitet werden soll.
     * @param mq                      MQ, für den die Tageswerte berechnet werden sollen
     * @param mapArchivIntervallDaten Map (Schlüssel Datum/Uhrzeit, Objekt Archivdatensatz) der Archivdaten für den MQ.
     *
     * @return Ergebnisdatensatz mit den DTVMonat, DTVQuartal oder DTVJahres-Werten
     */
    private ResultData berechneDTVMonatOderQuartalOderJahrMq(Auftrag auftrag, Mq mq, Map<Long, ArchiveData> mapArchivIntervallDaten) {
        Data            datenMq = _dav.createData(_atgMqDTV);  // Ergebnisdatensatz für den MQ
        DataDescription datenBeschreibung;                     // Datenbeschreibung des Ergebnisdatensatzes
        Data.Array      daMq;                                  // Hilfskonstruktion zum Zugriff auf die Arrays mit den Klassenwerten am MQ
        Parameter       p = Parameter.getInstanz();            // Instanz mit den Klassenparametern
        int             anz, i;                                // Hilfsvariablen
        int             anzahlIntervalleSoll = 0;              // Anzahl der Datensätze, die im fehlerfreien Fall zur Aggregation des Intervalls vorhanden sein sollten.

        // Aspekt abhängig vom Auftragstyp (IntervallDauer) setzen
        // Der Aspekt wird auf den Wert gesetzt, den man aggregiert, entspricht also dem Auftrag.
        // also bei Auftrag Monat --> Aspekt Tag etc.
        //
        // Zusätzlich wird abhängig vom Auftragstyp die anzahlIntervalleSoll ermittelt
        Aspect asp = null;

        switch (auftrag.getIntervallDauer()) {
        case DTV_MONAT :
            asp                  = _aspDTVMonat;
            anzahlIntervalleSoll = (int) ((auftrag.getIntervallEnde() - auftrag.getIntervallBeginn()) / (24 * 60 * 60 * 1000));  // Die Anzahl der Tage des Monats

            break;

        case DTV_QUARTAL :
            asp                  = _aspDTVQuartal;
            anzahlIntervalleSoll = 3;                                                                                            // Nämlich genau die 3 Monate des Quartals

            break;

        case DTV_JAHR :
            asp                  = _aspDTVJahr;
            anzahlIntervalleSoll = 4;                                                                                            // Nämlich genau die 4 Quartale des Jahres

            break;
        }

        datenBeschreibung = new DataDescription(_atgMqDTV, asp);

        // -----------------------------------------------------------------------------------------------
        // Initialisierung der Daten
        // -----------------------------------------------------------------------------------------------
        String[] attributNamenQWerte        = { "QKfzNk", "QKrad", "QPkw", "QLfw", "QPkwA", "QLkw", "QLkwA", "QSattelKfz", "QBus", "QSGV" };
        String[] attributNamenProzentWerte  = { "AnteilSGV" };
        String[] attributNamenVWerte        = { "VLVo", "VSGV", "VBPA", "SLVo", "SSGV", "SBPA", "V85LVo", "V85SGV", "V85BPA", "V15LVo", "V15SGV", "V15BPA" };
        String[] attributNamenQWerteKlassen = { "QLVoGeschwKlasse", "QSGVGeschwKlasse", "QBPAGeschwKlasse" };

        // QLVoGeschwKlasse vorbelegen
        anz = p.getGrenzenLVO().length + 1;
        datenMq.getArray("QLVoGeschwKlasse").setLength(anz);

        // QSGVGeschwKlasse vorbelegen
        anz = p.getGrenzenSGV().length + 1;
        datenMq.getArray("QSGVGeschwKlasse").setLength(anz);

        // QBPAGeschwKlasse vorbelegen
        anz = p.getGrenzenBPA().length + 1;
        datenMq.getArray("QBPAGeschwKlasse").setLength(anz);

        // VGrenzLVo vorbelegen (wird fix aus den Paramtern vorbelegt, da Grenzen beim Ergebnisdatensatz vorhanden sind.
        anz  = p.getGrenzenLVO().length;
        daMq = datenMq.getArray("VGrenzLVo");
        daMq.setLength(anz);

        for (i = 0; i < anz; i++) {
            daMq.getUnscaledValue(i).set(p.getGrenzenLVO()[i]);
        }

        // VGrenzSGV vorbelegen (wird fix aus den Paramtern vorbelegt, da Grenzen beim Ergebnisdatensatz vorhanden sind.
        anz  = p.getGrenzenSGV().length;
        daMq = datenMq.getArray("VGrenzSGV");
        daMq.setLength(anz);

        for (i = 0; i < anz; i++) {
            daMq.getUnscaledValue(i).set(p.getGrenzenSGV()[i]);
        }

        // VGrenzBPA vorbelegen (wird fix aus den Paramtern vorbelegt, da Grenzen beim Ergebnisdatensatz vorhanden sind.
        anz  = p.getGrenzenBPA().length;
        daMq = datenMq.getArray("VGrenzBPA");
        daMq.setLength(anz);

        for (i = 0; i < anz; i++) {
            daMq.getUnscaledValue(i).set(p.getGrenzenBPA()[i]);
        }

        // -----------------------------------------------------------------------------------------------
        // Berechnung des Ergebnisdatensatzes
        // -----------------------------------------------------------------------------------------------
        // Q-Werte berechnen
        for (String attributName : attributNamenQWerte) {
            datenMq.getUnscaledValue(attributName).set(aggregiereQWertAusArchivDaten(mapArchivIntervallDaten, attributName, true, anzahlIntervalleSoll));
        }

        // V-Werte berechnen
        for (String attributName : attributNamenVWerte) {
            datenMq.getUnscaledValue(attributName).set(aggregiereVWertAusArchivDaten(mapArchivIntervallDaten, attributName, true));
        }

        // %-Werte berechen
        for (String attributName : attributNamenProzentWerte) {
            datenMq.getUnscaledValue(attributName).set(aggregiereProzentWertAusArchivDaten(mapArchivIntervallDaten, attributName, true));
        }

        // Q-Klassenwerte berechnen
        for (String attributName : attributNamenQWerteKlassen) {
            daMq = datenMq.getArray(attributName);

            long[] QWertKlasse = aggregiereQWertKlassenAusArchivDaten(mapArchivIntervallDaten, attributName, true, anzahlIntervalleSoll);

            for (i = 0; i < daMq.getLength(); i++) {
                daMq.getUnscaledValue(i).set(QWertKlasse[i]);
            }
        }

        // -----------------------------------------------------------------------------------------------
        // Rückgabe des Ergebnisdatensatzen
        // -----------------------------------------------------------------------------------------------
        return new ResultData(mq.getMq(), datenBeschreibung, auftrag.getIntervallBeginn(), datenMq);
    }

    /**
     * Berechnet den Ergebnisdatensatz mit den DTVTag-Werten für einen MQ aus den FS-Werten des Tages
     *
     * @param auftrag                       Auftrag (für den Tag), der bearbeitet werden soll.
     * @param mq                            MQ, für den die Tageswerte berechnet werden sollen
     * @param listeMapsArchivIntervallDaten Liste mit einer Map (Schlüssel Datum/Uhrzeit, Objekt Archivdatensatz) der
     *                                      Archivdaten für die einzelnen FS des MQ. Je FS ist ein Listeneintrag
     *                                      vorhanden. Jeder Listeneintrag besteht aus einer zeitlich sortierten Map der
     *                                      FS-Daten für einen Tag.
     * @param listeMapsArchivIntervallDatenV2004
     *                                      Liste mit einer Map (Schlüssel Datum/Uhrzeit, Objekt Archivdatensatz) der
     *                                      Archivdaten der atg.verkehrsDatenLangzeitIntervall (BASt-V2004) für die
     *                                      einzelnen FS des MQ. Je FS ist ein Listeneintrag vorhanden. Jeder
     *                                      Listeneintrag besteht aus einer zeitlich sortierten Map der FS-Daten für
     *                                      einen Tag.
     *
     * @return Ergebnisdatensatz mit den DTVTag-Werten für einen MQ aus den FS-Werten des Tages
     */
    private ResultData berechneDTVTagMq(Auftrag auftrag, Mq mq, List<Map<Long, ArchiveData>> listeMapsArchivIntervallDaten, List<Map<Long, ArchiveData>> listeMapsArchivIntervallDatenV2004) {
        Data            datenMq           = _dav.createData(_atgMqDTV);                  // Ergebnisdatensatz für den MQ
        DataDescription datenBeschreibung = new DataDescription(_atgMqDTV, _aspDTVTag);  // Datenbeschreibung des Ergebnisdatensatzes
        Data.Array      daMq;                                                            // Hilfskonstruktion zum Zugriff auf die Arrays mit den Klassenwerten am MQ
        Parameter       p = Parameter.getInstanz();                                      // Instanz mit den Klassenparametern
        int             anz, i;                                                          // Hilfsvariablen

        // -----------------------------------------------------------------------------------------------
        // Initialisierung der Daten
        // -----------------------------------------------------------------------------------------------
        String[] attributNamenQWerte        = { "QKfzNk", "QKrad", "QPkw", "QLfw", "QPkwA", "QLkw", "QLkwA", "QSattelKfz", "QBus", "QSGV" };
        String[] attributNamenQWerteQKfz    = { "QKfzNk", "QKrad", "QPkw", "QLfw", "QPkwA", "QLkw", "QLkwA", "QSattelKfz", "QBus" };
        String[] attributNamenProzentWerte  = { "AnteilSGV" };
        String[] attributNamenVWerte        = { "VLVo", "VSGV", "VBPA", "SLVo", "SSGV", "SBPA", "V85LVo", "V85SGV", "V85BPA", "V15LVo", "V15SGV", "V15BPA" };
        String[] attributNamenQWerteKlassen = { "QLVoGeschwKlasse", "QSGVGeschwKlasse", "QBPAGeschwKlasse" };

        // QLVoGeschwKlasse vorbelegen
        anz = p.getGrenzenLVO().length + 1;
        datenMq.getArray("QLVoGeschwKlasse").setLength(anz);

        // QSGVGeschwKlasse vorbelegen
        anz = p.getGrenzenSGV().length + 1;
        datenMq.getArray("QSGVGeschwKlasse").setLength(anz);

        // QBPAGeschwKlasse vorbelegen
        anz = p.getGrenzenBPA().length + 1;
        datenMq.getArray("QBPAGeschwKlasse").setLength(anz);

        // VGrenzLVo vorbelegen (wird fix aus den Paramtern vorbelegt, da Grenzen beim Ergebnisdatensatz vorhanden sind.
        anz  = p.getGrenzenLVO().length;
        daMq = datenMq.getArray("VGrenzLVo");
        daMq.setLength(anz);

        for (i = 0; i < anz; i++) {
            daMq.getUnscaledValue(i).set(p.getGrenzenLVO()[i]);
        }

        // VGrenzSGV vorbelegen (wird fix aus den Paramtern vorbelegt, da Grenzen beim Ergebnisdatensatz vorhanden sind.
        anz  = p.getGrenzenSGV().length;
        daMq = datenMq.getArray("VGrenzSGV");
        daMq.setLength(anz);

        for (i = 0; i < anz; i++) {
            daMq.getUnscaledValue(i).set(p.getGrenzenSGV()[i]);
        }

        // VGrenzBPA vorbelegen (wird fix aus den Paramtern vorbelegt, da Grenzen beim Ergebnisdatensatz vorhanden sind.
        anz  = p.getGrenzenBPA().length;
        daMq = datenMq.getArray("VGrenzBPA");
        daMq.setLength(anz);

        for (i = 0; i < anz; i++) {
            daMq.getUnscaledValue(i).set(p.getGrenzenBPA()[i]);
        }

        // -----------------------------------------------------------------------------------------------
        // Berechnung des Ergebnisdatensatzes
        // -----------------------------------------------------------------------------------------------
        int anzFs = listeMapsArchivIntervallDaten.size();
        int anzVerfuegbareDaten, anzFehlendeDaten;

        // Q-Werte berechnen für den MQ
        //
        // ACHTUNG: Hier wird unterschieden, ob die reimportierten Daten der BASt V2004 bzgl. der q-Werte
        // berücksichtigt werden sollen
        if (!_isAggregationAusErfasstenDaten && _isQWerteAusReimportAtgVerkehrsDatenLangzeitIntervall) {

            // q-Werte aus BASt-V2004 Reimport sollen berücksichtigt werden
            for (String attributName : attributNamenQWerte) {

                // q-Werte für den Tag der FS berechnen
                long[] qWert = new long[anzFs];
                long   QWert = 0;

                anzVerfuegbareDaten = 0;
                anzFehlendeDaten    = 0;
                i                   = 0;

                for (Map<Long, ArchiveData> fsArchivDaten : listeMapsArchivIntervallDatenV2004) {
                    qWert[i] = aggregiereQWertAusArchivDatenV2004(fsArchivDaten, attributName, 24 / _intervallDauerInStunden);

                    if (qWert[i] == -1) {
                        anzFehlendeDaten++;
                    } else {
                        anzVerfuegbareDaten++;
                        QWert += qWert[i];
                    }

                    i++;
                }

                // QWert auf Basis der fehlenden Daten korrigieren
                if (anzVerfuegbareDaten == 0) {
                    QWert = -1;
                } else {
                    QWert = Math.round((float) QWert * (1.0 + ((float) anzFehlendeDaten / (float) anzVerfuegbareDaten)));
                }

                // Jetzt den Ergebniswert in den Datensatz für den MQ-Tageswert schreiben
                datenMq.getUnscaledValue(attributName).set(QWert);
            }
        } else {

            // q-Werte aus BASt-V2004 Reimport sollen NICHT berücksichtigt werden
            for (String attributName : attributNamenQWerte) {

                // q-Werte für den Tag der FS berechnen
                long[] qWert = new long[anzFs];
                long   QWert = 0;

                anzVerfuegbareDaten = 0;
                anzFehlendeDaten    = 0;
                i                   = 0;

                for (Map<Long, ArchiveData> fsArchivDaten : listeMapsArchivIntervallDaten) {
                    qWert[i] = aggregiereQWertAusArchivDaten(fsArchivDaten, attributName, false, 24 / _intervallDauerInStunden);

                    if (qWert[i] == -1) {
                        anzFehlendeDaten++;
                    } else {
                        anzVerfuegbareDaten++;
                        QWert += qWert[i];
                    }

                    i++;
                }

                // QWert auf Basis der fehlenden Daten korrigieren
                if (anzVerfuegbareDaten == 0) {
                    QWert = -1;
                } else {
                    QWert = Math.round((float) QWert * (1.0 + ((float) anzFehlendeDaten / (float) anzVerfuegbareDaten)));
                }

                // Jetzt den Ergebniswert in den Datensatz für den MQ-Tageswert schreiben
                datenMq.getUnscaledValue(attributName).set(QWert);
            }
        }

        // V-Werte berechnen
        for (String attributName : attributNamenVWerte) {

            // v-Werte für den Tag der FS berechnen
            int[] vWert = new int[anzFs];
            int   VWert = 0;

            anzVerfuegbareDaten = 0;
            anzFehlendeDaten    = 0;
            i                   = 0;

            for (Map<Long, ArchiveData> fsArchivDaten : listeMapsArchivIntervallDaten) {
                vWert[i] = aggregiereVWertAusArchivDaten(fsArchivDaten, attributName, false);

                if (vWert[i] == -1) {
                    anzFehlendeDaten++;
                } else {
                    anzVerfuegbareDaten++;
                    VWert += vWert[i];
                }

                i++;
            }

            // VWert auf Basis der fehlenden Daten korrigieren
            if (anzVerfuegbareDaten == 0) {
                VWert = -1;
            } else {
                VWert = Math.round((float) VWert / (float) anzVerfuegbareDaten);
            }

            // Jetzt den Ergebniswert in den Datensatz für den MQ-Tageswert schreiben
            datenMq.getUnscaledValue(attributName).set(VWert);
        }

        // %-Werte berechnen
        for (String attributName : attributNamenProzentWerte) {

            // Prozent-Werte für den Tag der FS berechnen
            long[] prozentWert = new long[anzFs];
            long   ProzentWert = 0;

            anzVerfuegbareDaten = 0;
            anzFehlendeDaten    = 0;
            i                   = 0;

            for (Map<Long, ArchiveData> fsArchivDaten : listeMapsArchivIntervallDaten) {
                prozentWert[i] = aggregiereProzentWertAusArchivDaten(fsArchivDaten, attributName, false);

                if (prozentWert[i] == -100001) {
                    anzFehlendeDaten++;
                } else {
                    anzVerfuegbareDaten++;
                    ProzentWert += prozentWert[i];
                }

                i++;
            }

            // QWert auf Basis der fehlenden Daten korrigieren
            if (anzVerfuegbareDaten == 0) {
                ProzentWert = -100001;
            } else {
                ProzentWert = Math.round((float) ProzentWert / (float) anzVerfuegbareDaten);
            }

            // Jetzt den Ergebniswert in den Datensatz für den MQ-Tageswert schreiben
            if (attributName.equals("AnteilSGV") && _isQWerteAusReimportAtgVerkehrsDatenLangzeitIntervall) {

                // AnteilSQV aus den bereits berechneten MQ-Q-Werten ermitteln
                long QKfz = 0;

                for (String qName : attributNamenQWerteQKfz) {
                    QKfz += Math.max(0, datenMq.getUnscaledValue(qName).longValue());
                }

                long QSGV = datenMq.getUnscaledValue("QSGV").longValue();

                ProzentWert = -100001;

                if (QKfz > 0) {
                    ProzentWert = (long) (((100.0 * (double) QSGV) / (double) QKfz) + 0.5);
                }
            }

            datenMq.getUnscaledValue(attributName).set(ProzentWert);
        }

        // Q-Klassenwerte berechnen
        for (String attributName : attributNamenQWerteKlassen) {

            // q-Klassen-Werte für den Tag der FS berechnen
            daMq = datenMq.getArray(attributName);

            int      anzKlassen                 = daMq.getLength();
            long[][] qWert                      = new long[anzFs][anzKlassen];
            long[]   QWert                      = new long[anzKlassen];
            int[]    anzVerfuegbareDatenKlassen = new int[anzKlassen];
            int[]    anzFehlendeDatenKlassen    = new int[anzKlassen];

            i = 0;

            for (Map<Long, ArchiveData> fsArchivDaten : listeMapsArchivIntervallDaten) {
                qWert[i] = aggregiereQWertKlassenAusArchivDaten(fsArchivDaten, attributName, false, 24 / _intervallDauerInStunden);

                for (int klasse = 0; klasse < anzKlassen; klasse++) {
                    if (qWert[i][klasse] == -1) {
                        anzFehlendeDatenKlassen[klasse]++;
                    } else {
                        anzVerfuegbareDatenKlassen[klasse]++;
                        QWert[klasse] += qWert[i][klasse];
                    }
                }

                i++;
            }

            // QWert auf Basis der fehlenden Daten korrigieren
            for (int klasse = 0; klasse < anzKlassen; klasse++) {
                if (anzVerfuegbareDatenKlassen[klasse] == 0) {
                    QWert[klasse] = -1;
                } else {
                    QWert[klasse] = Math.round((float) QWert[klasse] * (1.0 + ((float) anzFehlendeDatenKlassen[klasse] / (float) anzVerfuegbareDatenKlassen[klasse])));
                }
            }

            // Jetzt den Ergebniswert in den Datensatz für den MQ-Tageswert schreiben
            for (int klasse = 0; klasse < daMq.getLength(); klasse++) {
                daMq.getUnscaledValue(klasse).set(QWert[klasse]);
            }
        }

        // -----------------------------------------------------------------------------------------------
        // Rückgabe des Ergebnisdatensatzen
        // -----------------------------------------------------------------------------------------------
        return new ResultData(mq.getMq(), datenBeschreibung, auftrag.getIntervallBeginn(), datenMq);
    }

    /**
     * Ermittelt den Ergebnisdatensatz für MQ1, MQ2 und LZZS für die MSV-Werte des Jahres.
     *
     * @param auftrag       Der Auftrag, für den die Intervalldaten berechnet werden sollen.
     * @param so            Systemobjekt (MQ oder LZZS) für den der Datensatz berechnet wird.
     * @param mapDaten      Die Daten (Schlussel = Zeit, Stunde) und QKfzGesamt-Werte @param objektKennung
     *                      <code>0</code>=LZZS, <code>1</code>=MQ1 und <code>2</code>=MQ2
     * @param objektKennung 0 = LZZS, 1=MQ1, 2=MQ2
     *
     * @return Ergebnisdatensatz für die ATG <code>atg.verkehrsDatenLangZeitMSV</code>
     */
    private ResultData berechneMSV(Auftrag auftrag, SystemObject so, Map<Long, QKfzGesamtStundenWert> mapDaten, int objektKennung) {
        Data            datenMsv          = _dav.createData(_atgMSV);                   // Ergebnisdatensatz für den MQ
        DataDescription datenBeschreibung = new DataDescription(_atgMSV, _aspDTVJahr);  // Datenbeschreibung des Ergebnisdatensatzes

        // Neue Sortierte Map bilden, bei der Schlussel der QKfzGesamt-Wert für MQ1, MQ2 oder LZZS ist und die
        // Daten jeweils die Liste der Zeitstempel bilden
        Map<Long, List<Long>> sortierteListe = new TreeMap<Long, List<Long>>(new Comparator<Long>() {
            @Override

            // Sortierung muss absteigend erfolgen, also größter Wert zurerst!!
            public int compare(Long o1, Long o2) {
                if (o1 < o2) {
                    return 1;
                }

                if (o1 > o2) {
                    return -1;
                }

                return 0;
            }
        });

        for (long zeitPunkt : mapDaten.keySet()) {
            QKfzGesamtStundenWert qKfzGesamtStundenWert = mapDaten.get(zeitPunkt);

            // Erst mal prüfen, ob für den qKfzGesamtStundenWert schon ein Eintrag in der sortierten
            // Liste existiert, sonst einen neuen Eintrag anlegen
            long wert = qKfzGesamtStundenWert.getWert(objektKennung);

            // 0-Werte werden nicht berücksichtigt, da dies die Initialwerte sind
            if (wert > 0) {
                List<Long> listeZeitPunkte = sortierteListe.get(wert);

                if (listeZeitPunkte == null) {
                    listeZeitPunkte = new ArrayList<Long>();
                }

                // Zeitpunkt in Liste eintragen
                listeZeitPunkte.add(zeitPunkt);
                sortierteListe.put(wert, listeZeitPunkte);
            }
        }

        // Jetzt einfach den 1, 30 und 50 Eintrag der Liste einschließlich der Zeitstempel in den Ergebnisdatensatz schreiben.
        // Wenn es den jeweiligen Eintrag gar nicht gibt, dann Wert auf -1 setzten.
        // Dazu den Tree in eine Array umkopieren.
        long[] arrayWerte = new long[sortierteListe.size()];
        int    i          = 0;

        for (long wert : sortierteListe.keySet()) {
            arrayWerte[i] = wert;
            i++;
        }

        long qKfzGesamt_1  = -1;
        long qKfzGesamt_30 = -1;
        long qKfzGesamt_50 = -1;

        if (arrayWerte.length > 0) {
            qKfzGesamt_1 = arrayWerte[0];
            datenMsv.getUnscaledValue("01SpitzenStundeQKfzGesamt").set(qKfzGesamt_1);

            List<Long> zeitPunkte = sortierteListe.get(qKfzGesamt_1);

            Collections.sort(zeitPunkte);
            datenMsv.getTimeArray("01SpitzenStundeQKfzGesamtZeitPunkte").setLength(zeitPunkte.size());

            int j = 0;

            for (long zeitPunkt : zeitPunkte) {
                datenMsv.getTimeArray("01SpitzenStundeQKfzGesamtZeitPunkte").getTimeValue(j).setMillis(zeitPunkt);
                j++;
            }
        } else {
            datenMsv.getUnscaledValue("01SpitzenStundeQKfzGesamt").set(qKfzGesamt_1);
            datenMsv.getTimeArray("01SpitzenStundeQKfzGesamtZeitPunkte").setLength(0);
        }

        if (arrayWerte.length > 29) {
            qKfzGesamt_30 = arrayWerte[29];
            datenMsv.getUnscaledValue("30SpitzenStundeQKfzGesamt").set(qKfzGesamt_30);

            List<Long> zeitPunkte = sortierteListe.get(qKfzGesamt_30);

            Collections.sort(zeitPunkte);
            datenMsv.getTimeArray("30SpitzenStundeQKfzGesamtZeitPunkte").setLength(zeitPunkte.size());

            int j = 0;

            for (long zeitPunkt : zeitPunkte) {
                datenMsv.getTimeArray("30SpitzenStundeQKfzGesamtZeitPunkte").getTimeValue(j).setMillis(zeitPunkt);
                j++;
            }
        } else {
            datenMsv.getUnscaledValue("30SpitzenStundeQKfzGesamt").set(qKfzGesamt_30);
            datenMsv.getTimeArray("30SpitzenStundeQKfzGesamtZeitPunkte").setLength(0);
        }

        if (arrayWerte.length > 49) {
            qKfzGesamt_50 = arrayWerte[49];
            datenMsv.getUnscaledValue("50SpitzenStundeQKfzGesamt").set(qKfzGesamt_50);

            List<Long> zeitPunkte = sortierteListe.get(qKfzGesamt_50);

            Collections.sort(zeitPunkte);
            datenMsv.getTimeArray("50SpitzenStundeQKfzGesamtZeitPunkte").setLength(zeitPunkte.size());

            int j = 0;

            for (long zeitPunkt : zeitPunkte) {
                datenMsv.getTimeArray("50SpitzenStundeQKfzGesamtZeitPunkte").getTimeValue(j).setMillis(zeitPunkt);
                j++;
            }
        } else {
            datenMsv.getUnscaledValue("50SpitzenStundeQKfzGesamt").set(qKfzGesamt_50);
            datenMsv.getTimeArray("50SpitzenStundeQKfzGesamtZeitPunkte").setLength(0);
        }

        return new ResultData(so, datenBeschreibung, auftrag.getIntervallBeginn(), datenMsv);
    }

    /**
     * Sendesteuerung des Datenverteilers an die Applikation. Diese Methode muss von der Applikation implementiert
     * werden, um den Versand von Daten zu starten bzw. anzuhalten. Der Datenverteiler signalisiert damit einer Quelle
     * oder einem Sender dass mindestens ein Abnehmer bzw. kein Abnehmer mehr für die zuvor angemeldeten Daten vorhanden
     * ist. Die Quelle wird damit aufgefordert den Versand von Daten zu starten bzw. zu stoppen.
     *
     * @param object          Das in der zugehörigen Sendeanmeldung angegebene Objekt, auf das sich die Sendesteuerung
     *                        bezieht.
     * @param dataDescription Beschreibende Informationen zu den angemeldeten Daten auf die sich die Sendesteuerung
     *                        bezieht.
     * @param state           Status der Sendesteuerung. Kann einen der Werte <code>START_SENDING</code>,
     *                        <code>STOP_SENDING</code>, <code>STOP_SENDING_NO_RIGHTS</code>,
     *                        <code>STOP_SENDING_NOT_A_VALID_SUBSCRIPTION</code> enthalten.
     *
     * @see #START_SENDING
     * @see #STOP_SENDING
     * @see #STOP_SENDING_NO_RIGHTS
     * @see #STOP_SENDING_NOT_A_VALID_SUBSCRIPTION
     */
    @Override
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {

        // Wird nicht unterstützt. Ergebnisdaten werden immer gesendet.
    }

    /**
     * Ermittelt zu einer LZZS die MQ (an den Auswertequerschnitten) und die den MQ zugeordneten Fahrstreifen.
     *
     * @param lzzs Langzeitzählstellenobjekt, dessen MQ und FS ermittelt werden sollen.
     *
     * @return Liste der {@link de.inovat.dua.aggrlzzsdtv.VewLZZS.Mq}-Objekte an dieser LZZS.
     */
    private List<Mq> ermittleMq(SystemObject lzzs) {
        List<Mq>     listeMq = new ArrayList<Mq>();
        SystemObject soAwqInRichtung, soAwqGegenRichtung, soMqInRichtung, soMqGegenRichtung;

        // ATG an der LZZS mit den Referenzen auf die AWQ ermitteln
        AttributeGroup atg   = _konfiguration.getAttributeGroup("atg.langZeitZählstelle");
        Aspect         asp   = _konfiguration.getAspect("asp.eigenschaften");
        Data[]         daten = _konfiguration.getConfigurationData(new SystemObject[] { lzzs }, atg, asp);

        // AWQ InRichung ermitteln
        soAwqInRichtung = daten[0].getItem("InRichtung").getReferenceValue("Auswertequerschnitt").getSystemObject();

        // AWQ GegenRichung ermitteln
        soAwqGegenRichtung = daten[0].getItem("GegenRichtung").getReferenceValue("Auswertequerschnitt").getSystemObject();

        // MQ InRichtung ermitteln
        atg            = _konfiguration.getAttributeGroup("atg.auswerteQuerschnitt");
        asp            = _konfiguration.getAspect("asp.eigenschaften");
        daten          = _konfiguration.getConfigurationData(new SystemObject[] { soAwqInRichtung }, atg, asp);
        soMqInRichtung = daten[0].getReferenceValue("Messquerschnitt").getSystemObject();

        Mq mq = new Mq(soMqInRichtung);

        listeMq.add(mq);

        // MQ InRichtung ermitteln
        daten             = _konfiguration.getConfigurationData(new SystemObject[] { soAwqGegenRichtung }, atg, asp);
        soMqGegenRichtung = daten[0].getReferenceValue("Messquerschnitt").getSystemObject();
        mq                = new Mq(soMqGegenRichtung);
        listeMq.add(mq);

        // Liste mit den beiden MQ zurückgeben
        return listeMq;
    }

    /**
     * Ermittelt aus einem Archivdatensatz der ATG atg.verkehrsDatenLangZeitIntervallGeschwindigKeitsVerteilungFS die
     * Summer alle q-Werte unter Berücksichtigung nicht ermittelbarer Werte.
     *
     * @param ad Archivdatensatz der ATG atg.verkehrsDatenLangZeitIntervallGeschwindigKeitsVerteilungFS
     *
     * @return Summer alle q-Werte unter Berücksichtigung nicht ermittelbarer Werte
     */
    private long ermittleQKfzGesamt(ArchiveData ad) {
        final String[] attributeNamenQWerte = { "qKfzNk", "qKrad", "qPkw", "qLfw", "qPkwA", "qLkw", "qLkwA", "qSattelKfz", "qBus" };
        Data           daten                = ad.getData();
        long           qKfzGesamt           = -1;

        for (String att : attributeNamenQWerte) {
            long wert = daten.getUnscaledValue(att).longValue();

            if (wert > 0) {
                if (qKfzGesamt >= 0) {
                    qKfzGesamt += wert;
                } else {
                    qKfzGesamt = wert;
                }
            }
        }

        return qKfzGesamt;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert die Liste der MQ an der LZZZS.
     *
     * @return Liste der MQ an der LZZZS.
     */
    public List<Mq> getListeMq() {
        return _listeMq;
    }

    /**
     * Liefert das dieser Verwaltungseinheit zugeordnete LZZS Systemobjekt.
     *
     * @return Das dieser Verwaltungseinheit zugeordnete LZZS Systemobjekt.
     */
    public SystemObject getLzzs() {
        return _lzzs;
    }

    /**
     * Liefert die Pid des dieser Verwaltungseinheit zugeordnete LZZS Systemobjekt.
     *
     * @return Pid des dieser Verwaltungseinheit zugeordnete LZZS Systemobjekt.
     */
    public String getPid() {
        return _lzzs.getPid();
    }

    /**
     * Diese Methode muss von der Applikation implementiert werden, um zu signalisieren, ob Sendesteuerungen erwünscht
     * sind und mit der Methode <code>dataRequest</code> verarbeitet werden. In der Implementierung dieser Methode
     * dürfen keine synchronen Aufrufe, die auf Telegramme vom Datenverteiler warten (wie z.B. Konfigurationsanfragen)
     * durchgeführt werden, da ansonsten ein Deadlock entsteht.
     *
     * @param object          Das in der zugehörigen Sendeanmeldung angegebene System-Objekt.
     * @param dataDescription Die in der zugehörigen Sendeanmeldung angegebenen beschreibenden Informationen der
     *                        angemeldeten Daten.
     *
     * @return <code>true</code>, falls Sendesteuerungen gewünscht sind, sonst <code>false</code>.
     *
     * @see #dataRequest
     */
    @Override
    public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
        return false;
    }

    //~ INNERE KLASSEN ========================================================

    /** Hilfsklasse, die einen MQ (mit seinen FS) an einer LZZS verwaltet. */
    class Mq implements ClientSenderInterface {
        List<Fs>     _listeFs = new ArrayList<Fs>();
        SystemObject _mq;

        //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==========

        /**
         * Konstruktor der Verwaltungs Klasse.
         *
         * @param mq Das MQ-Systemojekt am LZZS
         */
        public Mq(SystemObject mq) {
            _mq = mq;

            // Menge mit den FS am MQ ermitteln
            List<SystemObject> listeSoFS = ((ConfigurationObject) mq).getNonMutableSet("FahrStreifen").getElements();

            for (SystemObject so : listeSoFS) {
                Fs fs = new Fs(so);

                _listeFs.add(fs);
            }

            // Sendeanmeldung für Zielattributgruppen
            try {
                _dav.subscribeSender(this, _mq, new DataDescription(_atgMqDTV, _aspDTVTag), SenderRole.source());
                _dav.subscribeSender(this, _mq, new DataDescription(_atgMqDTV, _aspDTVMonat), SenderRole.source());
                _dav.subscribeSender(this, _mq, new DataDescription(_atgMqDTV, _aspDTVQuartal), SenderRole.source());
                _dav.subscribeSender(this, _mq, new DataDescription(_atgMqDTV, _aspDTVJahr), SenderRole.source());
                _dav.subscribeSender(this, _mq, new DataDescription(_atgMSV, _aspDTVJahr), SenderRole.source());
            }
            catch (OneSubscriptionPerSendData e) {
                debug.warning(String.format("Doppelanmeldung für [%s] (andere Quelle vorhanden?). Statistikdaten werden für Objekt nicht weiter berücksichtigt.", _lzzs.getPid()), e);
            }
        }

        //~ METHODEN ==========================================================

        /**
         * Sendesteuerung des Datenverteilers an die Applikation. Diese Methode muss von der Applikation implementiert
         * werden, um den Versand von Daten zu starten bzw. anzuhalten. Der Datenverteiler signalisiert damit einer
         * Quelle oder einem Sender dass mindestens ein Abnehmer bzw. kein Abnehmer mehr für die zuvor angemeldeten
         * Daten vorhanden ist. Die Quelle wird damit aufgefordert den Versand von Daten zu starten bzw. zu stoppen.
         *
         * @param object          Das in der zugehörigen Sendeanmeldung angegebene Objekt, auf das sich die
         *                        Sendesteuerung bezieht.
         * @param dataDescription Beschreibende Informationen zu den angemeldeten Daten auf die sich die Sendesteuerung
         *                        bezieht.
         * @param state           Status der Sendesteuerung. Kann einen der Werte <code>START_SENDING</code>,
         *                        <code>STOP_SENDING</code>, <code>STOP_SENDING_NO_RIGHTS</code>,
         *                        <code>STOP_SENDING_NOT_A_VALID_SUBSCRIPTION</code> enthalten.
         *
         * @see #START_SENDING
         * @see #STOP_SENDING
         * @see #STOP_SENDING_NO_RIGHTS
         * @see #STOP_SENDING_NOT_A_VALID_SUBSCRIPTION
         */
        @Override
        public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {

            // Wird nicht unterstützt. Ergebnisdaten werden immer gesendet.
        }

        //~ GET METHODEN ======================================================

        /**
         * Liefert die Liste der {@link de.inovat.dua.aggrlzzsdtv.VewLZZS.Mq.Fs}-Objekte an diesem Mq.
         *
         * @return Liste der {@link de.inovat.dua.aggrlzzsdtv.VewLZZS.Mq.Fs}-Objekte an diesem Mq.
         */
        public List<Fs> getListeFs() {
            return _listeFs;
        }

        /**
         * Liefert das Systemobjekt des Mq.
         *
         * @return Das Systemobjekt des Mq.
         */
        public SystemObject getMq() {
            return _mq;
        }

        /**
         * Diese Methode muss von der Applikation implementiert werden, um zu signalisieren, ob Sendesteuerungen
         * erwünscht sind und mit der Methode <code>dataRequest</code> verarbeitet werden. In der Implementierung dieser
         * Methode dürfen keine synchronen Aufrufe, die auf Telegramme vom Datenverteiler warten (wie z.B.
         * Konfigurationsanfragen) durchgeführt werden, da ansonsten ein Deadlock entsteht.
         *
         * @param object          Das in der zugehörigen Sendeanmeldung angegebene System-Objekt.
         * @param dataDescription Die in der zugehörigen Sendeanmeldung angegebenen beschreibenden Informationen der
         *                        angemeldeten Daten.
         *
         * @return <code>true</code>, falls Sendesteuerungen gewünscht sind, sonst <code>false</code>.
         *
         * @see #dataRequest
         */
        @Override
        public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
            return false;
        }

        //~ INNERE KLASSEN ====================================================

        protected class Fs {
            SystemObject _fs;

            //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ======

            /**
             * Konstruktor der Verwaltungs Klasse.
             *
             * @param fs Das FS-Systemojekt am MQ
             */
            public Fs(SystemObject fs) {
                _fs = fs;
            }

            //~ GET METHODEN ==================================================

            /**
             * Liefert das Fahrstreifen Systemobjekt.
             *
             * @return Das Fahrstreifen Systemobjekt.
             */
            public SystemObject getFs() {
                return _fs;
            }
        }
    }


    /** Innere Klasse zur Verwaltung der QKfzGesamt-Werte für MQ1, MQ2 und LZZS */
    class QKfzGesamtStundenWert {
        long _wertLzzs = 0;
        long _wertMq1  = 0;
        long _wertMq2  = 0;

        //~ GET METHODEN ======================================================

        // Zum Testen mit Archiv aber ohne Archivdaten Werte per Zufall initialisieren
        // long _wertMq1  = (long) (Math.random() * 1000);
        // long _wertMq2  = (long) (Math.random() * 1000);
        // long _wertLzzs = _wertMq1 + _wertMq2;
        public long getWert(int objektKennung) {
            switch (objektKennung) {
            case 0 :
                return _wertLzzs;

            case 1 :
                return _wertMq1;

            case 2 :
                return _wertMq2;
            }

            return -1;
        }

        //~ METHODEN ==========================================================

        /**
         * Addiert einen gültigen qKfzGesamt-Fs-Wert auf den jeweiligen MQ-Wert auf und berechnet immer direkt auch den
         * LZZS-Wert
         *
         * @param indexMq    <code>1</code> für MQ1 und <code>2</code> für MQ2
         * @param qKfzGesamt Der qKfzGesamt-Wert am Fahrstreifen
         */
        public void setzeMqWert(int indexMq, long qKfzGesamt) {
            if (qKfzGesamt >= 0) {
                switch (indexMq) {
                case 1 :
                    _wertMq1 += qKfzGesamt;

                    break;

                case 2 :
                    _wertMq2 += qKfzGesamt;

                    break;
                }

                _wertLzzs = _wertMq1 + _wertMq2;
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 21.08.13
